/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include "bitboard.h"
#include "gen_moves.h"
#include "make_moves.h"
#include "attacks.h"
#include <chess_const.h>
#include <init.h>
#include <math_util.h>

BitBoard gen_attacks(char side){
  BitBoard a;
  BitBoard b;
  BitBoard to_board = 0;
  unsigned char key = 0;
  int from;
  if(side == WHITE){
    a = WKnights;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from = get_first_lsb(a);
      a^=mask[from];
      to_board |= knight_moves[from];
    }
    a = WBishops | WQueens;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      /* first the odd diagonal */
    /* Now i need to get the key that identifies the diagonal*/
      b = game.all_odd_diag;
      b = b >> odd_diag_shift[from];
      key = b & n_bit_mask[len_odd_diag[from]];   
      to_board |= bishop_odd_moves[from][(int)key];
      /* and now the even diagonal */
      b = game.all_even_diag;
      b = b >> even_diag_shift[from];
      key = b & n_bit_mask[len_even_diag[from]];   
      to_board |= bishop_even_moves[from][(int)key];
    }
    a = WRooks | WQueens;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      /* first the moves along the same rank*/
      b = game.b_pieces|game.w_pieces;
      b = b >> r_0_shift[from];
      key = b & n_bit_mask[8];   
      to_board |= rook_rank_moves[from][(int)key];
      /* and now along a file  */
      b = game.all_r_90;
      b = b >> r_90_shift[from];
      key = b & n_bit_mask[8];   
      to_board |= rook_file_moves[from][(int)key];
    }
    to_board |= (((WPawns & ~mask_file[7])>>9) & game.b_pieces);
    to_board |= (((WPawns & ~mask_file[0])>>7) & game.b_pieces);
    if((((WPawns & ~mask_file[7])>>9)|((WPawns & ~mask_file[0])>>7)) & mask[(int)game.en_passant]){
      to_board |= mask[(int)game.en_passant-8];
    }
    to_board |= king_moves[(int)game.white_king];
  }else{
    a = BKnights;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      to_board |= knight_moves[from];
    }
    a = BBishops | BQueens;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      /* first the odd diagonal */
    /* Now i need to get the key that identifies the diagonal*/
      b = game.all_odd_diag;
      b = b >> odd_diag_shift[from];
      key = b & n_bit_mask[len_odd_diag[from]];   
      to_board |= bishop_odd_moves[from][(int)key];
      /* and now the even diagonal */
      b = game.all_even_diag;
      b = b >> even_diag_shift[from];
      key = b & n_bit_mask[len_even_diag[from]];   
      to_board |= bishop_even_moves[from][(int)key];
    }
    a = BRooks | BQueens;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      /* first the moves along the same rank*/
      b = game.b_pieces|game.w_pieces;
      b = b >> r_0_shift[from];
      key = b & n_bit_mask[8];   
      to_board |= rook_rank_moves[from][(int)key];
      /* and now along a file  */
      b = game.all_r_90;
      b = b >> r_90_shift[from];
      key = b & n_bit_mask[8];   
      to_board |= rook_file_moves[from][(int)key];
    }
    to_board |= (((BPawns & ~mask_file[0])<<9) & game.w_pieces);
    to_board |= (((BPawns & ~mask_file[7])<<7) & game.w_pieces);
    if(game.en_passant && (((BPawns & ~mask_file[7])<<7)|((BPawns & ~mask_file[0])<<9)) & mask[(int)game.en_passant]){
      to_board |= mask[(int)game.en_passant+8];
    }
    to_board |= king_moves[(int)game.black_king];
  }
  return to_board;
}


/************************************************
 *                                              *
 *  Generates a bitboard containing all pieces  *
 *  attacking sq except en passant attacks      *
 *                                              *   
 ************************************************/

BitBoard gen_attackers(int sq){
  BitBoard ret = 0;

  ret = knight_moves[sq] & (WKnights | BKnights);
  ret |= (attacks_odd_diag(sq) | attacks_even_diag(sq)) & (WBishops | WQueens | BBishops | BQueens);
  ret |= (attacks_rank(sq) | attacks_file(sq)) & (WRooks | WQueens | BRooks | BQueens);
  ret |= ((mask[sq] & ~mask_file[0])<< 9 | (mask[sq] & ~mask_file[7]) << 7) & WPawns;
  ret |= ((mask[sq] & ~mask_file[7])>> 9 | (mask[sq] & ~mask_file[0])>> 7) & BPawns;
  ret |= king_moves[sq] & (mask[game.white_king] | mask[game.black_king]);
  return ret;
}

BitBoard attacks_rank(char from){
  unsigned char key = 0;
  BitBoard b = 0;
  b = game.b_pieces|game.w_pieces;
  b = b >> r_0_shift[(int)from];
  key = b & n_bit_mask[8];   
  return rook_rank_moves[(int)from][(int)key];
}

BitBoard attacks_file(char from){
  unsigned char key = 0;
  BitBoard b = 0;
  b = game.all_r_90;
  b = b >> r_90_shift[(int)from];
  key = b & n_bit_mask[8];   
  return rook_file_moves[(int)from][(int)key];
}

BitBoard attacks_odd_diag(char from){
  unsigned char key = 0;
  BitBoard b = 0;
  b = game.all_odd_diag;
  b = b >> odd_diag_shift[(int)from];
  key = b & n_bit_mask[len_odd_diag[(int)from]];   
  return bishop_odd_moves[(int)from][(int)key];
}

BitBoard attacks_even_diag(char from){
  unsigned char key = 0;
  BitBoard b = 0;
  b = game.all_even_diag;
  b = b >> even_diag_shift[(int)from];
  key = b & n_bit_mask[len_even_diag[(int)from]];   
  return bishop_even_moves[(int)from][(int)key];
}



/* 
   This is very similar to gen_attackers but instead of generating only potencial attacks is also pawn advances
   and treats en passant differently like gen_moves (see below)
   WARNING: It does not generate castles!!
*/

BitBoard gen_movers(char side, char sq){
  BitBoard a;
  BitBoard b;
  BitBoard to_board = 0;
  BitBoard ret = 0;
  unsigned char key = 0;
  int from;
  BitBoard empty;
  empty = ~(game.b_pieces|game.w_pieces);
  if(side == WHITE){
    a = WKnights;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      to_board = knight_moves[from];
      if(to_board & mask[(int)sq]){
	ret |= mask[from];
      }
    }
    a = WBishops | WQueens;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      /* first the odd diagonal */
    /* Now i need to get the key that identifies the diagonal*/
      b = game.all_odd_diag;
      b = b >> odd_diag_shift[from];
      key = b & n_bit_mask[len_odd_diag[from]];   
      to_board = bishop_odd_moves[from][(int)key];
      /* and now the even diagonal */
      b = game.all_even_diag;
      b = b >> even_diag_shift[from];
      key = b & n_bit_mask[len_even_diag[from]];   
      to_board |= bishop_even_moves[from][(int)key];
      if(to_board & mask[(int)sq]){
	ret |= mask[from];
      }
    }
    a = WRooks | WQueens;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      /* first the moves along the same rank*/
      b = game.b_pieces|game.w_pieces;
      b = b >> r_0_shift[from];
      key = b & n_bit_mask[8];   
      to_board = rook_rank_moves[from][(int)key];
      /* and now along a file  */
      b = game.all_r_90;
      b = b >> r_90_shift[from];
      key = b & n_bit_mask[8];   
      to_board |= rook_file_moves[from][(int)key];
      if(to_board & mask[(int)sq]){
	ret |= mask[from];
      }
    }
    to_board = (((WPawns & ~mask_file[7])>>9) & game.b_pieces);
    if(to_board & mask[(int)sq]){
      ret |= mask[(int)sq+9];
    }
    to_board = (((WPawns & ~mask_file[0])>>7) & game.b_pieces);
    if(to_board & mask[(int)sq]){
      ret |= mask[(int)sq+7];
    }
    if(game.en_passant && game.en_passant == sq){
      to_board = (((WPawns & ~mask_file[0])>>7) & mask[(int)game.en_passant]);
      if(to_board){
	ret |= mask[(int)game.en_passant+7];
      }
      to_board = (((WPawns & ~mask_file[7])>>9) & mask[(int)game.en_passant]);
      if(to_board){
	ret |= mask[(int)game.en_passant+9];
      }
    }      

    to_board = (WPawns>>8)&empty;
    if(to_board & mask[(int)sq]){
      ret |= mask[(int)sq+8];
    }
    to_board = (WPawns>>16)&(empty & empty >> 8 & mask_rank[3]);
    if(to_board & mask[(int)sq]){
      ret |= mask[(int)sq+16];
    }      
    to_board = king_moves[game.white_king];
    if(to_board & mask[(int)sq]){
      ret |= mask[game.white_king];
    }
    return ret;
  }else{
    a = BKnights;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      to_board = knight_moves[from];
      if(to_board & mask[(int)sq]){
	ret |= mask[from];
      }
    }
    a = BBishops | BQueens;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      /* first the odd diagonal */
    /* Now i need to get the key that identifies the diagonal*/
      b = game.all_odd_diag;
      b = b >> odd_diag_shift[from];
      key = b & n_bit_mask[len_odd_diag[from]];   
      to_board = bishop_odd_moves[from][(int)key];
      /* and now the even diagonal */
      b = game.all_even_diag;
      b = b >> even_diag_shift[from];
      key = b & n_bit_mask[len_even_diag[from]];   
      to_board |= bishop_even_moves[from][(int)key];
      if(to_board & mask[(int)sq]){
	ret |= mask[from];
      }
    }
    a = BRooks | BQueens;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      /* first the moves along the same rank*/
      b = game.b_pieces|game.w_pieces;
      b = b >> r_0_shift[from];
      key = b & n_bit_mask[8];   
      to_board = rook_rank_moves[from][(int)key];
      /* and now along a file  */
      b = game.all_r_90;
      b = b >> r_90_shift[from];
      key = b & n_bit_mask[8];   
      to_board |= rook_file_moves[from][(int)key];
      if(to_board & mask[(int)sq]){
	ret |= mask[from];
      }
    }
    to_board = (((BPawns & ~mask_file[0])<<9) & game.w_pieces);
    if(to_board & mask[(int)sq]){
      ret |= mask[(int)sq-9];
    }
    to_board = (((BPawns & ~mask_file[7])<<7) & game.w_pieces);
    if(to_board & mask[(int)sq]){
      ret |= mask[(int)sq-7];
    }
    if(game.en_passant && game.en_passant == sq){
      to_board = (((BPawns & ~mask_file[0])<<9) & mask[(int)game.en_passant]);
      if(to_board){
	ret |= mask[(int)game.en_passant-9];
      }
      to_board = (((BPawns & ~mask_file[7])<<7) & mask[(int)game.en_passant]);
      if(to_board){
	ret |= mask[(int)game.en_passant-7];
      }
    }
    to_board = (BPawns<<8)&empty;
    if(to_board & mask[(int)sq]){
      ret |= mask[(int)sq-8];
    }   
    to_board = (BPawns<<16)&(empty & empty << 8 & mask_rank[4]);
    if(to_board & mask[(int)sq]){
      ret |= mask[(int)sq-16];
    }   
    to_board = king_moves[game.black_king];
    if(to_board & mask[(int)sq]){
      ret |= mask[game.black_king];
    }
    return ret;
  }
  return 0;
}


/* 
   Again This is very similar to gen_attacks but instead of generating only potencial attacks is also pawn advances
   and treats en passants differents (it returns the square that ends up occupied not the square that gets captured.
   WARNING: It does not generate castles!!
*/


BitBoard gen_moves(char side){
  BitBoard a;
  BitBoard b;
  BitBoard to_board = 0;
  unsigned char key = 0;
  int from;
  BitBoard empty;
  empty = ~(game.b_pieces|game.w_pieces);
  if(side == WHITE){
    a = WKnights;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      to_board |= knight_moves[from];
    }
    a = WBishops | WQueens;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      /* first the odd diagonal */
    /* Now i need to get the key that identifies the diagonal*/
      b = game.all_odd_diag;
      b = b >> odd_diag_shift[from];
      key = b & n_bit_mask[len_odd_diag[from]];   
      to_board |= bishop_odd_moves[from][(int)key];
      /* and now the even diagonal */
      b = game.all_even_diag;
      b = b >> even_diag_shift[from];
      key = b & n_bit_mask[len_even_diag[from]];   
      to_board |= bishop_even_moves[from][(int)key];
    }
    a = WRooks | WQueens;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      /* first the moves along the same rank*/
      b = game.b_pieces|game.w_pieces;
      b = b >> r_0_shift[from];
      key = b & n_bit_mask[8];   
      to_board |= rook_rank_moves[from][(int)key];
      /* and now along a file  */
      b = game.all_r_90;
      b = b >> r_90_shift[from];
      key = b & n_bit_mask[8];   
      to_board |= rook_file_moves[from][(int)key];
    }
    to_board |= (((WPawns & ~mask_file[7])>>9) & game.b_pieces);
    to_board |= (((WPawns & ~mask_file[0])>>7) & game.b_pieces);
    if( game.en_passant && (((WPawns & ~mask_file[7])>>9)|((WPawns & ~mask_file[0])>>7)) & mask[(int)game.en_passant]){
      to_board |= mask[(int)game.en_passant];
    }
    to_board |= (WPawns>>8)&empty;
    to_board |= (WPawns>>16)&(empty & empty >> 8 & mask_rank[3]);
    to_board |= king_moves[(int)game.white_king];
  }else{
    a = BKnights;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      to_board |= knight_moves[from];
    }
    a = BBishops | BQueens;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      /* first the odd diagonal */
    /* Now i need to get the key that identifies the diagonal*/
      b = game.all_odd_diag;
      b = b >> odd_diag_shift[from];
      key = b & n_bit_mask[len_odd_diag[from]];   
      to_board |= bishop_odd_moves[from][(int)key];
      /* and now the even diagonal */
      b = game.all_even_diag;
      b = b >> even_diag_shift[from];
      key = b & n_bit_mask[len_even_diag[from]];   
      to_board |= bishop_even_moves[from][(int)key];
    }
    a = BRooks | BQueens;
    while (a) {
      /*Maybe i should try to replace get_first_lsb with get_first_msb */
      from=get_first_lsb(a);
      a^=mask[from];
      /* first the moves along the same rank*/
      b = game.b_pieces|game.w_pieces;
      b = b >> r_0_shift[from];
      key = b & n_bit_mask[8];   
      to_board |= rook_rank_moves[from][(int)key];
      /* and now along a file  */
      b = game.all_r_90;
      b = b >> r_90_shift[from];
      key = b & n_bit_mask[8];   
      to_board |= rook_file_moves[from][(int)key];
    }
    to_board |= (((BPawns & ~mask_file[0])<<9) & game.w_pieces);
    to_board |= (((BPawns & ~mask_file[7])<<7) & game.w_pieces);
    if( game.en_passant && (((BPawns & ~mask_file[7])<<7)|((BPawns & ~mask_file[0])<<9)) & mask[(int)game.en_passant]){
      to_board |= mask[(int)game.en_passant];
    }
    to_board |= (BPawns<<8)&empty;
    to_board |= (BPawns<<16)&(empty & empty << 8 & mask_rank[4]);
    to_board |= king_moves[(int)game.black_king];
  }
  return to_board;
}


