/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

/*
  Crafty style opening book
*/

#include <stdio.h>
#include "bitboard.h"
#include "gen_moves.h"
#include "interface.h"
#include "make_moves.h"
#include "search.h"
#include "util.h"
#include "commands.h"
#include "book.h"
#include <chess_const.h>
#include <init.h>
#include <zobrist.h>
#include <math_util.h>
#include <controls.h>


static FILE * book_file;

BookPosition  book_buffer[BOOK_CLUSTER_SIZE];

An_Move book_moves[MAX_POSSIBLE_MOVES];

void init_book(){  
  book_file = fopen("book.bin","rb");
  if(!book_file){
    fprintf(logfile,"Unable to open book.bin\n"); 
  }else{
    fprintf(logfile,"Book sucessfully open\n");   
  }      
}

int use_book(An_Move * ret){
  Move stack[MAX_POSSIBLE_MOVES];
  Move * top = &stack[0];
  Move * floor = top;

  BitBoard crafty_zob = game.zob_key;
  BitBoard temp_hash_key;
  BitBoard  common;
  int i;
  unsigned int key;
  unsigned int cluster;
  int played;
  int test;
  int found = 0;
  int bm_index = 0;
  int t_played = 0;
  int random_n;

  if(game.to_move == BLACK)
    crafty_zob = game.zob_key ^ change_side_zob;

  if (g_control.moves_out_of_book > 3) return(0);
  test = crafty_zob>>49; /* this gets the 15 high bits of the hash*/
  if (book_file){
    fseek(book_file,test*sizeof(int),SEEK_SET);
    cross_fread_uint32(&key,book_file);
    if (key > 0) {
      fseek(book_file,key,SEEK_SET);
      cross_fread_uint32(&cluster,book_file);
      cross_fread_book_positions(book_buffer,cluster,book_file);
      /*      fread(book_buffer,sizeof(BookPosition),cluster,book_file);*/
    } else {
      cluster = 0;
    }
    if(!cluster)
      return 0;

    top = valid_moves(top);
    common=crafty_zob & ~n_bit_mask[48];
    while(top>floor){
      top--;
      /* this should get the 16 high order bits */
      make_move(*top);
       /* this should generate the entire hash key we're looking for in the file */
      
      if(game.to_move == BLACK)
	crafty_zob = game.zob_key ^ change_side_zob;
      else
	crafty_zob = game.zob_key;
      /* And now some crafty magic */
      temp_hash_key = (crafty_zob ^ wtm_random[(int)game.to_move]);
      temp_hash_key = (temp_hash_key & n_bit_mask[48]) | common;
      for (i=0;i<cluster;i++) {
        if (!(temp_hash_key ^ book_buffer[i].position)) {
          played=book_buffer[i].status_played&077777777;
	  if(played > 5){
	    found = 1;
	    book_moves[bm_index].m = *top;
	    book_moves[bm_index].s = played;
	    bm_index++;
	    t_played += played;
	  }
          break;
        }
      }
      undo_move(*top);
    }
  }
  if(found){
    srand(time(NULL));
    /* we need to choose one of the possible solutions */
    random_n = (random_32() % (unsigned int)t_played);
    i = 0;
    while(random_n >= 0){
      random_n -= book_moves[i].s;
      i++;
    }
    ret->m = book_moves[i-1].m;
    ret->s = book_moves[i-1].s;
  }
  return found;
}
