/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include "bitboard.h"
#include "gen_moves.h"
#include "search.h"
#include <util.h>
#include <chess_const.h>
#include "make_moves.h"
#include "validate.h"
#include "attacks.h"
#include "check.h"
#include <init.h>



int check_draw(char in_search){
  BitBoard * top;
  int count = 0;
  if(move_50_counter < 4 && (game.white_pawns || game.black_pawns)) return 0;
  /* we'll skip the top move */
  top = history_floor+move_50_counter-4;
  while(top >= history_floor){
    if(*top == *(history_floor+move_50_counter)){
      /* If we are in the search tree we will return
	 a draw if we find a single repetition */ 
      if(in_search){
	return 1;
      }else{
	count++;
      }
    }
    top -= 2;
  }
  if(!in_search && count >= 2){
    if(count > 2){
      warn("Count > 2 in check repetition!\n"); 
    }
    return 1;
  }
  if(move_50_counter > 99){
    return 1;
  }
  /* mate material check */
  if(game.white_pawns == 0 && game.black_pawns == 0 &&
     game.black_majors == 0 && game.white_majors == 0 &&
     game.black_minors < 2 && game.white_minors < 2){
    return 1;
  }
  
  return 0;
}
      
char in_check(int side){
  if(side == BLACK){
    if(knight_moves[game.black_king] & WKnights)
      return 1;
    if(((mask[game.black_king] & ~mask_file[0])<< 9 | (mask[game.black_king] & ~mask_file[7]) << 7) & WPawns)
      return 1;
    if((attacks_rank(game.black_king) | attacks_file(game.black_king)) & (WRooks | WQueens))
      return 1;
    if((attacks_odd_diag(game.black_king) | attacks_even_diag(game.black_king)) & (WBishops | WQueens))
      return 1;
    if(king_moves[game.black_king] & (mask[game.white_king]))
      return 1;
    return 0;
  }else{
    if(knight_moves[game.white_king] & BKnights)
      return 1;
    if(((mask[game.white_king] & ~mask_file[7])>> 9 | (mask[game.white_king] & ~mask_file[0])>> 7) & BPawns)
      return 1;
    if((attacks_rank(game.white_king) | attacks_file(game.white_king)) & (BRooks | BQueens))
      return 1;
    if((attacks_odd_diag(game.white_king) | attacks_even_diag(game.white_king)) & (BBishops | BQueens))
      return 1;
    if(king_moves[game.white_king] & (mask[game.black_king]))
      return 1;
    return 0;
  }
  return 0;
}

/* piece_sq contanis a piece which is pinned to the piece on square square */
BitBoard pinned_to_square(char piece_sq,char square){
  int direc;
  BitBoard a;
  if(game.board[(int)piece_sq] > 0){
    /* piece is white */
    if(path[(int)piece_sq][(int)square] & (game.w_pieces | game.b_pieces)){
      return 0;
    }
    direc = directions[(int)piece_sq][(int)square];
    if(direc == FILE_UP || direc == FILE_DOWN){
      if((a = (attacks_file(piece_sq) & ((BRooks | BQueens) & ~mask[(int)square] & ~path[(int)piece_sq][(int)square]) ))){
	return a;
      }
      return 0;
    }else if(direc == RANK_LEFT || direc == RANK_RIGHT){
      if((a = (attacks_rank(piece_sq) & ((BRooks | BQueens) & ~mask[(int)square] & ~path[(int)piece_sq][(int)square])))){
	return a;
      }
      return 0;
    }else if(direc == ODD_UP || direc == ODD_DOWN){
      if((a = (attacks_odd_diag(piece_sq) & ((BBishops | BQueens) & ~mask[(int)square] & ~path[(int)piece_sq][(int)square])))){
	return a;
      }
      return 0;
    }else if(direc == EVEN_UP || direc == EVEN_DOWN){
      if((a = (attacks_even_diag(piece_sq) & ((BBishops | BQueens) & ~mask[(int)square] & ~path[(int)piece_sq][(int)square])))){
	return a;
      }
      return 0;
    }
  }else{
    /* piece is black */
    if(path[(int)piece_sq][(int)square] & (game.w_pieces | game.b_pieces)){
      return 0;
    }
    direc = directions[(int)piece_sq][(int)square];
    if(direc == FILE_UP || direc == FILE_DOWN){
      if((a = (attacks_file(piece_sq) & ((WRooks | WQueens) & ~mask[(int)square] & ~path[(int)piece_sq][(int)square])))){
	return a;
      }
      return 0;
    }else if(direc == RANK_LEFT || direc == RANK_RIGHT){
      if((a = (attacks_rank(piece_sq) & ((WRooks | WQueens) & ~mask[(int)square] & ~path[(int)piece_sq][(int)square])))){
	return a;
      }
      return 0;
    }else if(direc == ODD_UP || direc == ODD_DOWN){
      if((a = (attacks_odd_diag(piece_sq) & ((WBishops | WQueens) & ~mask[(int)square] & ~path[(int)piece_sq][(int)square])))){
	return a;
      }
      return 0;
    }else if(direc == EVEN_UP || direc == EVEN_DOWN){
      if((a = (attacks_even_diag(piece_sq) & ((WBishops | WQueens) & ~mask[(int)square] & ~path[(int)piece_sq][(int)square])))){
	return a;
      }
      return 0;
    }
  } 
  return 0;
}
