/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/


#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include "bitboard.h"
#include <gen_moves.h>
#include "search.h"
#include "util.h"
#include "search.h"
#include "interface.h"
#include "communicate.h"
#include "make_moves.h"
#include "book.h"
#include "elturco.h"
#include "epd.h"
#include "commands.h"
#include <chess_const.h>
#include <init.h>
#include <controls.h>
#include <zobrist.h>



int is_command(char * s){
  int ret = 0;
  int dummy;
  int depth;
  FILE * out;
  char buffer[1024];
  Position tmp_pos;
  char tmp_to_move;
  double time_d;

  Move m;
  /*  printf("-%s-\n",s);*/
  if(!strcmp(s,"q") || !strcmp(s,"quit")){
    fprintf(stderr,"Quiting...\n");
    exit(0);
  }
  if(strstr(s,"accepted")){
    ret = 1;
    return ret;
  }
  if(strstr(s,"xboard")){
    /*change to xboard mode*/
    x_control.use_xboard = 1;
    x_control.silent = 1;
    ret = 1;
  }
  if(strstr(s,"protover 2")){
    printf("feature ping=1\n");
    printf("feature setboard=1\n");
    printf("feature playother=1\n");
    printf("feature san=1\n");
    printf("feature usermove=0\n");
    printf("feature time=1\n");
    printf("feature draw=0\n");
    printf("feature sigint=0\n");
    printf("feature sigterm=0\n");
    printf("feature reuse=1\n");
    printf("feature analyze=0\n");
    printf("feature myname=\"El Turco %s\"\n",VERSION);
    printf("feature variants=\"normal\"\n");
    printf("feature colors=0\n");
    printf("feature ics=0\n");
    printf("feature pause=0\n");
    printf("feature done=1\n");
    /* make sure silent is 1*/
    x_control.silent = 1;
    ret = 1;
  }
  if(strstr(s,"new")){
    if(game.to_move != game.comp){
      g_control.pass_move = 1;
      g_control.continue_game = 1;
    }
    mate_in = 0;
    init_book();  
    init_game(&game);  
    init_move_maker(&game);
    init_position = 1;
    g_control.force = 0;
    ret = 1;
  }
  if(strstr(s,"display")){
    print_board(stdout);
    ret = 1;
  }
  if(strstr(s,"random")){
    ret = 1;
  }
  if(strstr(s,"level")){
    sscanf(s,"level %d %d %d",&dummy,&(t_control.timeleft),&(t_control.increment));
    t_control.timeleft *= 60*100;
    t_control.increment *= 100;
    t_control.starting_time = t_control.timeleft;   
    ret = 1;
  }
  if(strstr(s,"undo")){
    ret = 1;
  }
  if(strstr(s,"hard")){    
    ret = 1;
  }
  if(strstr(s,"easy")){    
    ret = 1;
  }
  if(strstr(s,"st ")){    
    ret = 1;
  }
  if(strstr(s,"sd ")){    
    ret = 1;
  }
  if(strstr(s,"go")){
    g_control.force = 0;
    game.comp = game.to_move;
    g_control.pass_move = 1;
    g_control.continue_game = 1;
    ret = 1;
  }
  if(strstr(s,"playother")){
    g_control.force = 0;
    game.comp = !game.to_move;
    if(game.comp == game.to_move){
      g_control.pass_move = 1;
      g_control.continue_game = 1;
    }
    ret = 1;
  }
  if(strstr(s,"computer")){
    ret = 1;
  }
  if(strstr(s,"result")){
    sscanf(s,"result %s",buffer);
    fprintf(g_control.pgn,"%s\n",buffer);
    ret = 1;
  }
  if(strstr(s,"draw")){
    ret = 1;
  }
  if(strstr(s,"hint")){
    ret = 1;
  }
  if(strstr(s,"variant")){
    ret = 1;  }
  if(strstr(s,"rating")){
    sscanf(s,"rating %d %d",&g_control.elo[0],&g_control.elo[1]);
    ret = 1;
  }
  if(strstr(s,"name")){
    sscanf(s,"name %s",g_control.opponent);
    ret = 1;
  }
  if(strstr(s,"otim")){
    ret = 1;
  }
  if(strstr(s,"time")){
    t_control.fixed_time = 0;
    sscanf(s,"time %d",&(t_control.timeleft));
    ret = 1;
  }
  /* Don't change the order of post and nopost or we really screw things up!
   This is so close to a bug it's disturing...
  */
  if(strstr(s,"post")){
    x_control.thinking = 1;
    ret = 1;
  }
  if(strstr(s,"nopost")){
    x_control.thinking = 0;
    ret = 1;
  }
  if(strstr(s,"ics ")){
    ret = 1;
  }
  if(strstr(s,"white")){
    game.comp = BLACK;
    if(game.comp == game.to_move){
      g_control.pass_move = 1;
      if(!g_control.force)
	g_control.continue_game = 1;
    }
    ret = 1;
  }
  if(strstr(s,"black")){
    game.comp = WHITE;
    if(game.comp == game.to_move){
      g_control.pass_move = 1;
      if(!g_control.force)
	g_control.continue_game = 1;
    }
    ret = 1;
  }
  if(strstr(s,"force")){
    g_control.force = 1;
    ret = 1;
  }
  if(strstr(s,"?")){
    if(g_control.phase == SEARCHING)
      end_search();
    ret = 1;/* this only gets executed if not searching otherwise stop_search will jump to search()*/
  }
  if(strstr(s,"setboard")){
    /* s+9 so i don't send the "setboard "part */
    load_board(s+9);
    ret = 1;
  }
  /* EPD support */
  if(strstr(s,"epd")){
    if(strlen(s) > 3 && sscanf(s,"epd %s",buffer) > 0){
      out = fopen(buffer,"a");
      run_epd_testsuite(out);
    }else{
      run_epd_testsuite(stdout);
    }
    ret = 1;
  }
  /* perftest support*/
  if(strstr(s,"perft")){
    if(sscanf(s,"perft %d",&depth) != 1){
      depth = 5;
    }
    tmp_pos = game;
    tmp_to_move = game.comp;
    game.comp = game.to_move;
    /*    if(game.comp != tmp_to_move){
      game.zob_key ^= change_side_zob;
      }*/
    time_d = get_time();
    dummy = perft(depth);
    time_d = get_time() - time_d;
    printf("Perft depth %d - %d moves\nTime - %fs\n",depth,dummy,time_d);
    game = tmp_pos;
    ret = 1;
  }

  if(strncmp(s,"ping ",4) == 0){
    /* replace ping by pong */
    s[1] = 'o';
    printf("%s\n",s);
    ret = 1;
  }
  if(g_control.force){
    m = algebraic2move(s,&game);
    if(m.from != m.to){
      init_position = 0;
      make_move(m);
      printf("Making forced move\n");
      ret = 1;
    }
  }  
  return ret;
}
