/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include <time.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include "bitboard.h"
#include <gen_moves.h>
#include "search.h"
#include "commands.h"
#include "util.h"
#include "communicate.h"
#include <chess_const.h>
#include <controls.h>

#ifdef _WIN32
#include <conio.h>
#include <windows.h>
#include <winbase.h>
#include <wincon.h>
#include <io.h>
#endif


/*
  Borrowed from pepito, which seems borrowed from crafty
  Thank god for opensource
*/
int poll_input(){
#ifndef _WIN32
    static fd_set   readfds;
    struct timeval  tv;
    int             ret;

    FD_ZERO(&readfds);
    FD_SET(fileno(stdin), &readfds);
    tv.tv_sec = 0;
    tv.tv_usec = 0;
    select(16, &readfds, NULL, NULL, &tv);
    ret = FD_ISSET(fileno(stdin), &readfds);
    return (ret);
#else


    static int      init = 0,
                    pipe;
    static HANDLE   inh;
    static INPUT_RECORD Buffer[256];
    DWORD           dw=0;

    if (!init) {
        init = 1;
        inh = GetStdHandle(STD_INPUT_HANDLE);
        pipe = !GetConsoleMode(inh, &dw);
        if (!pipe) {
            SetConsoleMode(inh, dw & ~(ENABLE_MOUSE_INPUT | ENABLE_WINDOW_INPUT));
            FlushConsoleInputBuffer(inh);
        }
    }
    if (pipe) {
        if (!PeekNamedPipe(inh, NULL, 0, NULL, &dw, NULL))
            return 1;
        return dw;
    } else {
        GetNumberOfConsoleInputEvents(inh, &dw);
        return dw <= 1 ? 0 : dw;
    }
#endif
}

char * get_input(int block){
  char * buffer = NULL;
  FILE * log;
  char filename[30];
  int i;
  if(block){
    buffer = (char *)malloc(sizeof(char)*1000);
    fgets(buffer,1000,stdin);
    sprintf(filename,"comlog%d.txt",getpid());
    log = fopen(filename,"a");
    fprintf(log,"%s",buffer);
    fclose(log);
    /* remove crap from buffer */
    chomp(buffer);    
    if(is_command(buffer)){
      free(buffer);
      return NULL;
    }else{
      if(strlen(buffer)){
	return buffer;
      }else{
	return NULL;
      } 
    }
  }else{
    while(poll_input() || g_control.force){
      buffer = (char *)malloc(sizeof(char)*1000);
      i = 0;
      while(poll_input()){
	buffer[i] = getc(stdin);
	i++;
	/*	printf("I'm blocked!\n");*/
	/*      fgets(buffer,1000,stdin);*/
      /*      printf("I WAS blocked. ;-)\n");*/
      }
      sprintf(filename,"comlog%d.txt",getpid());
      log = fopen(filename,"a");
      fprintf(log,"%s",buffer);
      fclose(log);
      /* remove crap from buffer */
      chomp(buffer);
      
      if(is_command(buffer)){
	free(buffer);
	buffer = NULL;
      }else{
	if(strlen(buffer))
	  return buffer;
	else
	  return NULL;
      }
    }
    return NULL;
  }
}


