/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include <bitboard.h>
#include <gen_moves.h>
#include "search.h"
#include <util.h>

#include <consistency.h>
#include <math_util.h>
#include <init.h>

char check_boards_cons(){
  BitBoard a;
  int i;
  a = WPawns;
  while(a){
    i = get_first_lsb(a);
    a ^= mask[i];
    if(game.board[i] != w_pawn){
      error("w_pawn inconsistent");
      return 0;
    }
  }
  a = WKnights;
  while(a){
    i = get_first_lsb(a);
    a ^= mask[i];
    if(game.board[i] != w_knight){
      error("w_knight inconsistent");
      return 0;
    }
  }
  a = WBishops;
  while(a){
    i = get_first_lsb(a);
    a ^= mask[i];
    if(game.board[i] != w_bishop){
      error("w_bishop inconsistent");
      return 0;
    }
  }
  a = WQueens;
  while(a){
    i = get_first_lsb(a);
    a ^= mask[i];
    if(game.board[i] != w_queen){
      error("w_queen inconsistent");
      return 0;
    }
  }
  a = WRooks;
  while(a){
    i = get_first_lsb(a);
    a ^= mask[i];
    if(game.board[i] != w_rook){
      error("w_rook inconsistent");
      return 0;
    }
  }
  a = game.w_pieces;
  while(a){
    i = get_first_lsb(a);
    a ^= mask[i];
    if(game.board[i] <= 0){
      error("w_pieces inconsistent");
      return 0;
    }
  }
  a = BPawns;
  while(a){
    i = get_first_lsb(a);
    a ^= mask[i];
    if(game.board[i] != b_pawn){
      error("b_pawn inconsistent");
      return 0;
    }
  }
  a = BKnights;
  while(a){
    i = get_first_lsb(a);
    a ^= mask[i];
    if(game.board[i] != b_knight){
      error("b_knight inconsistent");
      return 0;
    }
  }
  a = BBishops;
  while(a){
    i = get_first_lsb(a);
    a ^= mask[i];
    if(game.board[i] != b_bishop){
      error("b_bishop inconsistent");
      return 0;
    }
  }
  a = BQueens;
  while(a){
    i = get_first_lsb(a);
    a ^= mask[i];
    if(game.board[i] != b_queen){
      error("b_queen inconsistent");
      return 0;
    }
  }
  a = BRooks;
  while(a){
    i = get_first_lsb(a);
    a ^= mask[i];
    if(game.board[i] != b_rook){
      error("b_rook inconsistent");
      return 0;
    }
  }
  a = game.b_pieces;
  while(a){
    i = get_first_lsb(a);
    a ^= mask[i];
    if(game.board[i] >= 0){
      error("b_pieces inconsistent");
      return 0;
    }
  }
  for(i = 0;i<64;i++){
    if(game.board[i]){
      if(!(game.all_r_90 & mask[r_90_map[i]]) ||
	 !(game.all_odd_diag & mask[odd_diag_map[i]]) ||
	 !(game.all_even_diag & mask[even_diag_map[i]])){
	error("rotated bitboards inconsistent");
      }
    }else{
      if((game.all_r_90 & mask[r_90_map[i]]) ||
	 (game.all_odd_diag & mask[odd_diag_map[i]]) ||
	 (game.all_even_diag & mask[even_diag_map[i]])){
	error("rotated bitboards inconsistent");
      }
    }
  }

      
  return 1;
}
