/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include "bitboard.h"
#include "gen_moves.h"
#include "search.h"
#include <controls.h>
#include "util.h"
#include "make_moves.h"
#include "interface.h"
#include "book.h"
#include <string.h>
#include "commands.h"
#include "communicate.h"
#include "check.h"
#include "book.h"
#include "elturco.h"
#include <init.h>
#include <score.h>
#include <chess_const.h>

/* TODO 
 Try to understand what EPD is and how it can be used for testing
*/


FILE * logfile;

int main(int argc, char ** argv){
  parse_args(argc,argv);
  open_log();
  init_const_boards();
  init_book();
  init_game(&game);  
  init_move_maker(&game);
  print_board(stdout);
  play();
  close_log();
  printf("Exiting...\n");
  return 0;
}

void play(){
  An_Move engine_result;
  Move opponent_move = {0,0,0,0};
  while(1){
    if(check_draw(0)){
      draw();
      continue;
    }

    if(game.to_move == game.comp){
      if(use_book(&engine_result)){
	g_control.moves_out_of_book = 0;
      }else{
	engine_result = search();
	if(engine_result.s <= -(MATE-MAX_DEPTH) || (g_control.quick_resign && engine_result.s <= -(ROOK_VALUE))){
	  resign();
	  continue;
	}
	g_control.moves_out_of_book++;
      }
      make_engine_move(engine_result);
      if(engine_result.m.special & IRREVERSIBLE){
	init_move_maker(&game);
      }
    }else{
      opponent_move = get_opponent_move();
      if(g_control.pass_move){
	g_control.pass_move = 0;
      }else{
	make_opponent_move(opponent_move);
      }
    }
  }  
}


void make_engine_move(An_Move best){
  if(x_control.use_xboard){
    printf("move ");
  }else if(!x_control.silent){
    printf("Move - ");
  }
  if(!g_control.move_nr){
    /* first move */
    print_pgn_header();
  }
    
    
  if((g_control.move_nr & 1) == 0){
    /* ply is even */
    fprintf(g_control.pgn,"%d.",g_control.move_nr/2+1);
  }
  fprintf(g_control.pgn,"%s ",move2algebraic(best.m,&game));
  fflush(g_control.pgn);
  printf("%s\n",move2algebraic(best.m,&game));
  make_move(best.m);
  g_control.move_nr++;
  if(!x_control.silent){
    printf("Score: %d\n",best.s);
    print_board(stdout);
  }
}

void print_position(){
  if(!x_control.silent){
    printf("Zob key: %llu\n",game.zob_key);
    print_board(stdout);
  }
}

void make_opponent_move(Move  m){
  if(!x_control.silent){
    printf("Move - ");
    printf("%s\n",move2algebraic(m,&game));
  }
  if(!g_control.move_nr){
    /* first move */
    print_pgn_header();
  }
  if((g_control.move_nr & 1) == 0){
    /* ply is even */
    fprintf(g_control.pgn,"%d.",g_control.move_nr/2+1);
  }
  fprintf(g_control.pgn,"%s ",move2algebraic(m,&game));
  fflush(g_control.pgn);
  make_move(m);
  g_control.move_nr++;
  print_position();
}

void resign(){
  game.to_move = !game.comp;
  if(!x_control.silent){
    printf("El Turco resigns\n");
  }else{
    printf("tellics resign\n");
    if(game.comp == WHITE)
      printf("0-1 {White resigns}\n");
    else
      printf("1-0 {Black resigns}\n");
  }	
}

void draw(){
  init_game(&game);  
  init_move_maker(&game);
  game.to_move = !game.comp;
  printf("1/2-1/2 {Draw}\n");
}
