/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include "bitboard.h"
#include <gen_moves.h>
#include <score.h>
#include "search.h"
#include "util.h"
#include "interface.h"
#include "tt.h"
#include "make_moves.h"
#include "epd.h"
#include <init.h>
#include <controls.h>

void setup_epd_line(char* inbuff)
{
  int i = 0;
  int j = 0;
  char tmp;
  /* find the end of the 4th field */
  while(i<4){
    if(inbuff[j] == ' ')
      i++;
    j++;
  }
  tmp = inbuff[j];
  inbuff[j] = 0;
  /* this does the real job */
  load_board(inbuff);
  inbuff[j] = tmp;
  init_tt();
  init_move_maker(&game);  
}

int check_solution(char *inbuff, An_Move res)
{
  int ret = 0;
  char * san;
  
  san = move2algebraic(res.m,&game);
 
  
  if (strstr(inbuff, "bm") != NULL){
    if (strstr(inbuff, san) != NULL)
      ret = 1;
    else
      ret = 0;
  } else if (strstr(inbuff, "am") != NULL){
    if (strstr(inbuff, san) != NULL)
      ret =  0;
    else
      ret = 1;
  } else {
    printf("No best-move or avoid-move found!\n");
  }
  free(san);
  return ret;
}

void run_epd_testsuite(FILE * output)
{
  FILE *testsuite;
  char readbuff[2000];
  char testname[FILENAME_MAX];
  double elapsed; 
  int nps;
  long thinktime;
  int tested, found;
  double time_i;
  double time_f;
  An_Move res;

  
  tested = 0;
  found = 0;
  
  printf("\nName of EPD testsuite: ");
  fgets(testname,FILENAME_MAX-1, stdin);
  /* remove crap from buffer */
  chomp(testname);

  printf("\nTime per move (s): ");
  fgets(readbuff,FILENAME_MAX, stdin);
  chomp(readbuff);
  thinktime = atol(readbuff);
  printf("\n");

  thinktime *= 100;
  t_control.fixed_time = thinktime;

  testsuite = fopen(testname, "r");

  while (fgets(readbuff, 2000, testsuite) != NULL) {
    fflush(output);
    tested++;
    setup_epd_line(readbuff);
    print_board(stdout);      
    time_i = get_time();
    res = search();
    time_f = get_time();
      
    fprintf (output,"\nScore: %f\n", res.s/100.0);
    fprintf (output,"Nodes: %ld (%0.2f%% qnodes)\n", total_nodes,
	    (float) ((float) quies_nodes / (float) total_nodes * 100.0));
      
    elapsed = time_f-time_i;
    nps = (int)((float) total_nodes/(float) elapsed);
      
    if (!elapsed){
      fprintf (output,"NPS: N/A\n");
    }else{
      fprintf (output,"NPS: %ld\n", (long int) nps);
      fprintf (output,": %ld\n", (long int) nps);
    }
    /* I need to add this statistics!! */
    /*
      printf("ECacheProbes : %ld   ECacheHits : %ld   HitRate : %f%%\n", 
      ECacheProbes, ECacheHits, 
      ((float)ECacheHits/((float)ECacheProbes+1)) * 100);
      
      printf("TTStores : %ld TTProbes : %ld   TTHits : %ld   HitRate : %f%%\n", 
      TTStores, TTProbes, TTHits, 
      ((float)TTHits/((float)TTProbes+1)) * 100);
      
      printf("NTries : %d  NCuts : %d  CutRate : %f%%  TExt: %d\n", 
      NTries, NCuts, (((float)NCuts*100)/((float)NTries+1)), TExt);
      
      printf("Check extensions: %ld  Razor drops : %ld  Razor Material : %ld\n", ext_check, razor_drop, razor_material);
      printf("EGTB Hits: %d  EGTB Probes: %d  Efficiency: %3.1f%%\n", EGTBHits, EGTBProbes,
      (((float)EGTBHits*100)/(float)(EGTBProbes+1)));		 
      
      printf("Move ordering : %f%%\n", (((float)FHF*100)/(float)FH+1));
      
      printf("Material score: %d   Eval : %d\n", Material, eval());
      printf("\n");
    */
    if(res.s < MATE){
      if(check_solution(readbuff, res))
	{
	  found++;
	  fprintf(output,"Solution found.\n");
	}
      else
	{
	  fprintf(output,"Solution not found.\n");
	}
    }else {
      found++;
    }	
    fprintf(output,"Solved: %d/%d\n", found, tested);	
  };  
  printf("\n");
  t_control.fixed_time = 0;
}

