/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include <gen_moves.h>
#include <bitboard.h>
#include <zobrist.h>
#include <chess_const.h>
#include <attacks.h>
#include "search.h"
#include <util.h>
#include <check.h>
#include <make_moves.h>
#include <init.h>
#include <math_util.h>


/* I'll use this nice idea i saw of using a stack to store 
   the moves. It receives a pointer to the top of the stack
   and returns the updated pointer to the top   */

Move * w_knight_cap(Move * top){
  BitBoard a = WKnights;
  int from,to;
  BitBoard to_board;
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    to_board = knight_moves[from]&game.b_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = CAPTURE;
      top->special |= IRREVERSIBLE;
      top++;
    }
  }
  return top;
}

Move * w_bishop_cap(Move * top){
  BitBoard a = WBishops;
  BitBoard b;
  int from,to;
  BitBoard to_board;
  unsigned char key;
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    /* first the odd diagonal */
    /* Now i need to get the key that identifies the diagonal*/
    b = game.all_odd_diag;
    b = b >> odd_diag_shift[from];
    key = b & n_bit_mask[len_odd_diag[from]];   
    to_board = bishop_odd_moves[from][(int)key]&game.b_pieces;
    /* and now the even diagonal */
    b = game.all_even_diag;
    b = b >> even_diag_shift[from];
    key = b & n_bit_mask[len_even_diag[from]];   
    to_board |= bishop_even_moves[from][(int)key]&game.b_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = CAPTURE;
      top->special |= IRREVERSIBLE;
      top++;
    }
  }
  return top;
}


Move * w_rook_cap(Move * top){
  BitBoard a = WRooks;
  BitBoard b;
  int from,to;
  BitBoard to_board;
  unsigned char key;
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    /* first the moves along the same rank*/
    b = game.b_pieces|game.w_pieces;
    b = b >> r_0_shift[from];
    key = b & n_bit_mask[8];   
    to_board = rook_rank_moves[from][(int)key]&game.b_pieces;
    /* and now along a file  */
    b = game.all_r_90;
    b = b >> r_90_shift[from];
    key = b & n_bit_mask[8];   
    to_board |= rook_file_moves[from][(int)key]&game.b_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = CAPTURE;
      top->special |= IRREVERSIBLE;
      top++;
    }
  }
  return top;
}

Move * w_queen_cap(Move * top){
  BitBoard a = WQueens;
  BitBoard b;
  int from,to;
  BitBoard to_board;
  unsigned char key;
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    /* first the moves along the same rank*/
    b = game.b_pieces|game.w_pieces;
    b = b >> r_0_shift[from];
    key = b & n_bit_mask[8];   
    to_board = rook_rank_moves[from][(int)key]&game.b_pieces;
    /* and now along a file  */
    b = game.all_r_90;
    b = b >> r_90_shift[from];
    key = b & n_bit_mask[8];   
    to_board |= rook_file_moves[from][(int)key]&game.b_pieces;
    /* and now the odd diagonal */
    b = game.all_odd_diag;
    b = b >> odd_diag_shift[from];
    key = b & n_bit_mask[len_odd_diag[from]];   
    to_board |= bishop_odd_moves[from][(int)key]&game.b_pieces;
    /* and now the even diagonal */
    b = game.all_even_diag;
    b = b >> even_diag_shift[from];
    key = b & n_bit_mask[len_even_diag[from]];   
    to_board |= bishop_even_moves[from][(int)key]&game.b_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = CAPTURE;
      top->special |= IRREVERSIBLE;
      top++;
    }
  }
  return top;
}

Move * w_pawn_cap(Move * top){
  int to;
  int i;
  BitBoard to_board;
  /* the pawn move generation is slightly different from the rest */
  if(!game.en_passant){    
    /* first lets check even diag captures */
    /* we have to mask the first file out to prevent 
       wrap around problems */
    to_board = ((WPawns & ~mask_file[7])>>9)&game.b_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      if(ROW(to)){
	top->to_promote=0;
	top->from=to+9;
	top->to=to;
	top->special = CAPTURE;
	top->special |= IRREVERSIBLE;
	top++;
      }else{
	for(i = 2;i<6;i++){
	  top->from=to+9;
	  top->to=to;
	  top->to_promote=i;
	  top->special = CAPTURE;
	  top->special |= IRREVERSIBLE;
	  top++;
	}
      }
    }
    /* and now odd diag captures */
    to_board |= ((WPawns & ~mask_file[0])>>7)&game.b_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      if(ROW(to)){
	top->to_promote=0;
	top->from=to+7;
	top->to=to;
	top->special = CAPTURE;
	top->special |= IRREVERSIBLE;
	top++;

      }else{
	for(i = 2;i<6;i++){
	  top->from=to+7;
	  top->to=to;
	  top->to_promote=i;
	  top->special = CAPTURE;
	  top->special |= IRREVERSIBLE;
	  top++;
	}
      }
    }
  }else{
    to_board = ((WPawns & ~mask_file[7])>>9)&(game.b_pieces | mask[(int)game.en_passant]);
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      if(ROW(to)){
	top->to_promote=0;
	top->from=to+9;
	top->to=to;
	top->special = CAPTURE;
	if(top->to == game.en_passant)
	  top->special |= EN_PASSANT;
	top->special |= IRREVERSIBLE;
	top++;	
      }else{
	for(i = 2;i<6;i++){
	  top->to_promote=i;
	  top->from=to+9;
	  top->to=to;
	  top->special = CAPTURE;
	  if(top->to == game.en_passant)
	    top->special |= EN_PASSANT;
	  top->special |= IRREVERSIBLE;
	  top++;
	}
      }
    }
    to_board = ((WPawns & ~mask_file[0])>>7)&(game.b_pieces | mask[(int)game.en_passant]);
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      if(ROW(to)){
	top->from=to+7;
	top->to=to;
	top->to_promote=0;
	top->special = CAPTURE;
	if(top->to == game.en_passant)
	  top->special |= EN_PASSANT;
	top->special |= IRREVERSIBLE;
	top++;	
      }else{
	for(i = 2;i<6;i++){
	  top->from=to+7;
	  top->to=to;
	  top->to_promote=i;
	  top->special = CAPTURE;
	  if(top->to == game.en_passant)
	    top->special |= EN_PASSANT;
	  top->special |= IRREVERSIBLE;
	  top++;	
	}
      }
    }
  }
  return top;
}

Move * w_king_cap(Move * top){
  int to;
  BitBoard to_board;
  to_board = king_moves[game.white_king]& game.b_pieces;
  while (to_board) {
    to=get_first_lsb(to_board);
    to_board^=mask[to];
    top->from=game.white_king;
    top->to=to;
    top->to_promote=0;
    top->special = CAPTURE;
    top->special |= IRREVERSIBLE;
    top++;
  }
  return top;
}


Move * w_pawn_promotions(Move * top){
  int to;
  int i;
  BitBoard to_board;
  if(WPawns & mask_rank[6]){
    to_board = ((WPawns & mask_rank[6])>>8) & ~(game.w_pieces|game.b_pieces);
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      for(i = 2;i<6;i++){
	top->from=to+8;
	top->to=to;
	top->to_promote=i;
	top->special = IRREVERSIBLE;
	top++;
      }
    }
    to_board = ((WPawns & mask_rank[6]& ~mask_file[7])>>9)&game.b_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      for(i = 2;i<6;i++){
	top->from=to+9;
	top->to=to;
	top->to_promote=i;
	top->special = CAPTURE;
	top->special |= IRREVERSIBLE;
	top++;
      }
    }
    to_board |= ((WPawns & mask_rank[6]& ~mask_file[0])>>7)&game.b_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      for(i = 2;i<6;i++){
	top->from=to+7;
	top->to=to;
	top->to_promote=i;
	top->special = CAPTURE;
	top->special |= IRREVERSIBLE;
	top++;
      }
    }

  }
  return top;
}


Move * all_w_cap(Move * top){
  top = w_knight_cap(top);
  top = w_bishop_cap(top);
  top = w_rook_cap(top);
  top = w_queen_cap(top);
  top = w_pawn_cap(top);
  top = w_king_cap(top);
  top = w_pawn_promotions(top);
  return top;
}


Move * all_w_cap_non_promotions(Move * top){
  top = w_knight_cap(top);
  top = w_bishop_cap(top);
  top = w_rook_cap(top);
  top = w_queen_cap(top);
  top = w_pawn_cap(top);
  top = w_king_cap(top);
  return top;
}


Move * w_knight_non_cap(Move * top){
  BitBoard a = WKnights;
  int from,to;
  BitBoard to_board;
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    to_board = knight_moves[from]& ~(game.b_pieces|game.w_pieces);
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = 0;
      top++;
    }
  }
  return top;
}

Move * w_bishop_non_cap(Move * top){
  BitBoard a = WBishops;
  BitBoard b;
  BitBoard empty;
  int from,to;
  BitBoard to_board;
  unsigned char key;
  empty = ~(game.b_pieces|game.w_pieces);
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    /* first the odd diagonal */
    /* Now i need to get the key that identifies the diagonal*/
    b = game.all_odd_diag;
    b = b >> odd_diag_shift[from];
    key = b & n_bit_mask[len_odd_diag[from]];   
    to_board = bishop_odd_moves[from][(int)key]&empty;
    /* and now the even diagonal */
    b = game.all_even_diag;
    b = b >> even_diag_shift[from];
    key = b & n_bit_mask[len_even_diag[from]];   
    to_board |= bishop_even_moves[from][(int)key]&empty;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = 0;
      top++;
    }
  }
  return top;
}


Move * w_rook_non_cap(Move * top){
  BitBoard a = WRooks;
  BitBoard b;
  int from,to;
  BitBoard to_board;
  BitBoard empty;
  unsigned char key;
  empty = ~(game.b_pieces|game.w_pieces);
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    /* first the moves along the same rank*/
    b = game.b_pieces|game.w_pieces;
    b = b >> r_0_shift[from];
    key = b & n_bit_mask[8];   
    to_board = rook_rank_moves[from][(int)key]&empty;
    /* and now along a file  */
    b = game.all_r_90;
    b = b >> r_90_shift[from];
    key = b & n_bit_mask[8];   
    to_board |= rook_file_moves[from][(int)key]&empty;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = 0;
      top++;
    }
  }
  return top;
}


Move * w_queen_non_cap(Move * top){
  BitBoard a = WQueens;
  BitBoard b;
  int from,to;
  BitBoard to_board;
  BitBoard empty;
  unsigned char key;
  empty = ~(game.b_pieces|game.w_pieces);
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    /* first the moves along the same rank*/
    b = game.b_pieces|game.w_pieces;
    b = b >> r_0_shift[from];
    key = b & n_bit_mask[8];   
    to_board = rook_rank_moves[from][(int)key]&empty;
    /* and now along a file  */
    b = game.all_r_90;
    b = b >> r_90_shift[from];
    key = b & n_bit_mask[8];   
    to_board |= rook_file_moves[from][(int)key]&empty;
    /* and now the odd diagonal */
    b = game.all_odd_diag;
    b = b >> odd_diag_shift[from];
    key = b & n_bit_mask[len_odd_diag[from]];   
    to_board |= bishop_odd_moves[from][(int)key]&empty;
    /* and now the even diagonal */
    b = game.all_even_diag;
    b = b >> even_diag_shift[from];
    key = b & n_bit_mask[len_even_diag[from]];   
    to_board |= bishop_even_moves[from][(int)key]&empty;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = 0;
      top++;
    }
  }
  return top;
}


Move * w_pawn_non_cap(Move * top){
  int to;
  int i;
  BitBoard to_board;
  BitBoard empty;
  empty = ~(game.b_pieces|game.w_pieces);
  to_board = (WPawns>>8)&empty;
  while (to_board) {
    to=get_first_lsb(to_board);
    to_board^=mask[to];
    if(ROW(to)){      
      top->from=to+8;
      top->to=to;
      top->to_promote=0;
      top->special = IRREVERSIBLE;
      top++;
    
    }else{
      for(i = 2;i<6;i++){
	top->from=to+8;
	top->to=to;
	top->to_promote=i;
	top->special = IRREVERSIBLE;
	top++;
      }
    }
  }

  to_board = (WPawns>>16)&(empty & empty >> 8 & mask_rank[3]);
  while (to_board) {
    to=get_first_lsb(to_board);
    to_board^=mask[to];
    top->from=to+16;
      top->to=to;
      top->to_promote=0;
      top->special = IRREVERSIBLE;
      top->special |= POSSIBLE_EN_PASSANT;
      top++;
  }
  return top;
}

Move * w_king_non_cap(Move * top){
  int to;
  BitBoard to_board;
  to_board = king_moves[game.white_king]& ~(game.b_pieces|game.w_pieces);
  while (to_board) {
    to=get_first_lsb(to_board);
    to_board^=mask[to];
    top->from=game.white_king;
    top->to=to;
    top->to_promote=0;
    top->special = 0;
    top++;
  }
  return top;
}


Move * w_castles(Move * top){
  Move stack[100];
  Move * my_top = &(stack[0]);
  int flag;
  if((game.new_castle & (W_SMALL_CASTLE | W_BIG_CASTLE)) == 0){
    return top;
  }else{
    if(game.new_castle & W_SMALL_CASTLE){
      /* lets check if we can castle right*/
      if(!game.board[62] && !game.board[61] && game.board[52] != b_pawn){
	/* the path is clear, now we must check for attacked squares */
	my_top = all_b_cap(my_top);
	my_top = all_b_non_cap_non_castle(my_top);
	flag = 0;
	while(my_top > &stack[0]){
	  my_top--;
	  if(my_top->to == E1 || my_top->to == F1 || my_top->to == G1){
	    flag = 1;
	    break;
	  }
	}
	if(flag == 0){
	  top->from = CASTLE_FROM;
	  top->to = CASTLE_TO;
	  top->special = 0;
	  top->special |= W_SMALL_CASTLE;
	  top->to_promote = 0;
	  top++;
	}
      }
    }
    if(game.new_castle & W_BIG_CASTLE){
      /* lets check if we can castle left*/
      if(!game.board[57] && !game.board[58] && !game.board[59] && game.board[52] != b_pawn){
	/* the path is clear */
	/* the path is clear, now we must check for attacked squares */
	my_top = &(stack[0]);
	my_top = all_b_cap(my_top);
	my_top = all_b_non_cap_non_castle(my_top);
	flag = 0;
	while(my_top > &stack[0]){
	  my_top--;
	  if(my_top->to == E1 || my_top->to == D1 || my_top->to == C1){
	    flag = 1;
	    break;
	  }
	}
	if(flag == 0){
	  top->from = CASTLE_FROM;
	  top->to = CASTLE_TO;
	  top->special = 0;
	  top->special |= W_BIG_CASTLE;
	  top->to_promote = 0;
	  top++;
	}
	
      }
    }
  }
  return top;
}

 

Move * all_w_non_cap(Move * top){
  top = w_castles(top);
  top = w_knight_non_cap(top);
  top = w_bishop_non_cap(top);
  top = w_rook_non_cap(top);
  top = w_queen_non_cap(top);
  top = w_pawn_non_cap(top);
  top = w_king_non_cap(top);
  return top;
}


Move * all_w_non_cap_non_castle(Move * top){
  top = w_knight_non_cap(top);
  top = w_bishop_non_cap(top);
  top = w_rook_non_cap(top);
  top = w_queen_non_cap(top);
  top = w_pawn_non_cap(top);
  /* this are pseudo legal moves */
  top = w_king_non_cap(top);
  return top;
}




Move * b_knight_cap(Move * top){
  BitBoard a = BKnights;
  int from,to;
  BitBoard to_board;
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    to_board = knight_moves[from]&game.w_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = CAPTURE;
      top->special |= IRREVERSIBLE;
      top++;
    }
  }
  return top;
}

Move * b_bishop_cap(Move * top){
  BitBoard a = BBishops;
  BitBoard b;
  int from,to;
  BitBoard to_board;
  unsigned char key;
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    /* first the odd diagonal */
    /* Now i need to get the key that identifies the diagonal*/
    b = game.all_odd_diag;
    b = b >> odd_diag_shift[from];
    key = b & n_bit_mask[len_odd_diag[from]];   
    to_board = bishop_odd_moves[from][(int)key]&game.w_pieces;
    /* and now the even diagonal */
    b = game.all_even_diag;
    b = b >> even_diag_shift[from];
    key = b & n_bit_mask[len_even_diag[from]];   
    to_board |= bishop_even_moves[from][(int)key]&game.w_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = CAPTURE;
      top->special |= IRREVERSIBLE;
      top++;
    }
  }
  return top;
}


Move * b_rook_cap(Move * top){
  BitBoard a = BRooks;
  BitBoard b;
  int from,to;
  BitBoard to_board;
  unsigned char key;
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    /* first the moves along the same rank*/
    b = game.b_pieces|game.w_pieces;
    b = b >> r_0_shift[from];
    key = b & n_bit_mask[8];   
    to_board = rook_rank_moves[from][(int)key]&game.w_pieces;
    /* and now along a file  */
    b = game.all_r_90;
    b = b >> r_90_shift[from];
    key = b & n_bit_mask[8];   
    to_board |= rook_file_moves[from][(int)key]&game.w_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = CAPTURE;
      top->special |= IRREVERSIBLE;
      top++;
    }
  }
  return top;
}

Move * b_queen_cap(Move * top){
  BitBoard a = BQueens;
  BitBoard b;
  int from,to;
  BitBoard to_board;
  unsigned char key;
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    /* first the moves along the same rank*/
    b = game.b_pieces|game.w_pieces;
    b = b >> r_0_shift[from];
    key = b & n_bit_mask[8];   
    to_board = rook_rank_moves[from][(int)key]&game.w_pieces;
    /* and now along a file  */
    b = game.all_r_90;
    b = b >> r_90_shift[from];
    key = b & n_bit_mask[8];   
    to_board |= rook_file_moves[from][(int)key]&game.w_pieces;
    /* and now the odd diagonal */
    b = game.all_odd_diag;
    b = b >> odd_diag_shift[from];
    key = b & n_bit_mask[len_odd_diag[from]];   
    to_board |= bishop_odd_moves[from][(int)key]&game.w_pieces;
    /* and now the even diagonal */
    b = game.all_even_diag;
    b = b >> even_diag_shift[from];
    key = b & n_bit_mask[len_even_diag[from]];   
    to_board |= bishop_even_moves[from][(int)key]&game.w_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = CAPTURE;
      top->special |= IRREVERSIBLE;
      top++;
    }
  }
  return top;
}

Move * b_pawn_cap(Move * top){
  int to;
  int i;
  BitBoard to_board;
  /* the pawn move generation is slightly different from the rest */
  if(!game.en_passant){    
    /* first lets check even diag captures */
    /* we have to mask the first file out to prevent 
       wrap around problems */
    to_board = ((BPawns & ~mask_file[0])<<9)&game.w_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      if(RANK(to)){
	top->from=to-9;
	top->to=to;
	top->to_promote=0;
	top->special = CAPTURE;
	top->special |= IRREVERSIBLE;
	top++;
      }else{
	for(i = 2;i<6;i++){
	  top->from=to-9;
	  top->to=to;
	  top->to_promote=i;
	  top->special = CAPTURE;
	  top->special |= IRREVERSIBLE;
	  top++;
	}
      }
    }
    /* and now odd diag captures */
    to_board |= ((BPawns & ~mask_file[7])<<7)&game.w_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      if(RANK(to)){
	top->from=to-7;
	top->to=to;
	top->to_promote=0;
	top->special = CAPTURE;
	top->special |= IRREVERSIBLE;
	top++;
      }else{
	for(i = 2;i<6;i++){
	  top->from=to-7;
	  top->to=to;
	  top->to_promote=i;
	  top->special = CAPTURE;
	  top->special |= IRREVERSIBLE;
	  top++;
	}
      }
    }
  }else{
    to_board = ((BPawns & ~mask_file[0])<<9)&(game.w_pieces | mask[(int)game.en_passant]);
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      if(RANK(to)){
	top->from=to-9;
	top->to=to;
	top->to_promote=0;
	top->special = CAPTURE;
	if(top->to == game.en_passant)
	  top->special |= EN_PASSANT;
	top->special |= IRREVERSIBLE;
	top++;
      }else{
	for(i = 2;i<6;i++){
	  top->from=to-9;
	  top->to=to;
	  top->to_promote=i;
	  top->special = CAPTURE;
	  if(top->to == game.en_passant)
	    top->special |= EN_PASSANT;
	  top->special |= IRREVERSIBLE;
	  top++;	  
	}
      }
    }
    to_board = ((BPawns & ~mask_file[7])<<7)&(game.w_pieces | mask[(int)game.en_passant]);
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      if(RANK(to)){
	top->from=to-7;
	top->to=to;	
	top->to_promote=0;
	top->special = CAPTURE;
	if(top->to == game.en_passant)
	  top->special |= EN_PASSANT;
	top->special |= IRREVERSIBLE;
	top++;
      }else{
	for(i = 2;i<6;i++){
	  top->from=to-7;
	  top->to=to;	
	  top->to_promote=i;
	  top->special = CAPTURE;
	  if(top->to == game.en_passant)
	    top->special |= EN_PASSANT;
	  top->special |= IRREVERSIBLE;
	  top++;
	}
      }
    }
  }
  return top;
}

Move * b_king_cap(Move * top){
  int to;
  BitBoard to_board;
  to_board = king_moves[game.black_king]& game.w_pieces;
  while (to_board) {
    to=get_first_lsb(to_board);
    to_board^=mask[to];
    top->from=game.black_king;
    top->to=to;
    top->to_promote=0;
    top->special = CAPTURE;
    top->special |= IRREVERSIBLE;
    top++;
  }
  return top;
}


Move * b_pawn_promotions(Move * top){
  int to;
  int i;
  BitBoard to_board;
  if(BPawns & mask_rank[1]){
    to_board = ((BPawns & mask_rank[1])<<8) & ~(game.w_pieces|game.b_pieces);
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      for(i = 2;i<6;i++){
	top->from=to-8;
	top->to=to;
	top->to_promote=i;
	top->special = IRREVERSIBLE;
	top++;
      }
    }
    to_board = ((BPawns & mask_rank[1]& ~mask_file[7])<<7)&game.w_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      for(i = 2;i<6;i++){
	top->from=to-7;
	top->to=to;
	top->to_promote=i;
	top->special = CAPTURE;
	top->special |= IRREVERSIBLE;
	top++;
      }
    }
    to_board = ((BPawns & mask_rank[1]& ~mask_file[0])<<9)&game.w_pieces;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      for(i = 2;i<6;i++){
	top->from=to-9;
	top->to=to;
	top->to_promote=i;
	top->special = CAPTURE;
	top->special |= IRREVERSIBLE;
	top++;
      }
    }
  }
  return top;
}

Move * all_b_cap(Move * top){
  top = b_knight_cap(top);
  top = b_bishop_cap(top);
  top = b_rook_cap(top);
  top = b_queen_cap(top);
  top = b_pawn_cap(top);
  top = b_king_cap(top);
  top = b_pawn_promotions(top);
  return top;
}

Move * all_b_cap_non_promotions(Move * top){
  top = b_knight_cap(top);
  top = b_bishop_cap(top);
  top = b_rook_cap(top);
  top = b_queen_cap(top);
  top = b_pawn_cap(top);
  top = b_king_cap(top);
  return top;
}


Move * b_knight_non_cap(Move * top){
  BitBoard a = BKnights;
  int from,to;
  BitBoard to_board;
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    to_board = knight_moves[from]& ~(game.b_pieces|game.w_pieces);
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = 0;
      top++;
    }
  }
  return top;
}

Move * b_bishop_non_cap(Move * top){
  BitBoard a = BBishops;
  BitBoard b;
  BitBoard empty;
  int from,to;
  BitBoard to_board;
  unsigned char key;
  empty = ~(game.b_pieces|game.w_pieces);
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    /* first the odd diagonal */
    /* Now i need to get the key that identifies the diagonal*/
    b = game.all_odd_diag;
    b = b >> odd_diag_shift[from];
    key = b & n_bit_mask[len_odd_diag[from]];   
    to_board = bishop_odd_moves[from][(int)key]&empty;
    /* and now the even diagonal */
    b = game.all_even_diag;
    b = b >> even_diag_shift[from];
    key = b & n_bit_mask[len_even_diag[from]];   
    to_board |= bishop_even_moves[from][(int)key]&empty;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = 0;
      top++;
    }
  }
  return top;
}


Move * b_rook_non_cap(Move * top){
  BitBoard a = BRooks;
  BitBoard b;
  int from,to;
  BitBoard to_board;
  BitBoard empty;
  unsigned char key;
  empty = ~(game.b_pieces|game.w_pieces);
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    /* first the moves along the same rank*/
    b = game.b_pieces|game.w_pieces;
    b = b >> r_0_shift[from];
    key = b & n_bit_mask[8];   
    to_board = rook_rank_moves[from][(int)key]&empty;
    /* and now along a file  */
    b = game.all_r_90;
    b = b >> r_90_shift[from];
    key = b & n_bit_mask[8];   
    to_board |= rook_file_moves[from][(int)key]&empty;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = 0;
      top++;
    }
  }
  return top;
}


Move * b_queen_non_cap(Move * top){
  BitBoard a = BQueens;
  BitBoard b;
  int from,to;
  BitBoard to_board;
  BitBoard empty;
  unsigned char key;
  empty = ~(game.b_pieces|game.w_pieces);
  while (a) {
    /*Maybe i should try to replace get_first_lsb with get_first_msb */
    from=get_first_lsb(a);
    a^=mask[from];
    /* first the moves along the same rank*/
    b = game.b_pieces|game.w_pieces;
    b = b >> r_0_shift[from];
    key = b & n_bit_mask[8];   
    to_board = rook_rank_moves[from][(int)key]&empty;
    /* and now along a file  */
    b = game.all_r_90;
    b = b >> r_90_shift[from];
    key = b & n_bit_mask[8];   
    to_board |= rook_file_moves[from][(int)key]&empty;
    /* and now the odd diagonal */
    b = game.all_odd_diag;
    b = b >> odd_diag_shift[from];
    key = b & n_bit_mask[len_odd_diag[from]];   
    to_board |= bishop_odd_moves[from][(int)key]&empty;
    /* and now the even diagonal */
    b = game.all_even_diag;
    b = b >> even_diag_shift[from];
    key = b & n_bit_mask[len_even_diag[from]];   
    to_board |= bishop_even_moves[from][(int)key]&empty;
    while (to_board) {
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      top->from=from;
      top->to=to;
      top->to_promote=0;
      top->special = 0;
      top++;
    }
  }
  return top;
}


Move * b_pawn_non_cap(Move * top){
  int to;
  int i;
  BitBoard to_board;
  BitBoard empty;
  empty = ~(game.b_pieces|game.w_pieces);
  to_board = (BPawns<<8)&empty;
  while (to_board) {
    to=get_first_lsb(to_board);
    to_board^=mask[to];
    if(RANK(to)){
      top->from=to-8;
      top->to=to;
      top->to_promote=0;
      top->special = IRREVERSIBLE;
      top++;

    }else{
      for(i = 2;i<6;i++){
	top->from=to-8;
	top->to=to;
	top->to_promote=i;
	top->special = IRREVERSIBLE;
	top++;
      }

    }
  }
  to_board = (BPawns<<16)&(empty & empty << 8 & mask_rank[4]);
  while (to_board) {
    to=get_first_lsb(to_board);
    to_board^=mask[to];
    top->from=to-16;
    top->to=to;
    top->to_promote=0;
    top->special = IRREVERSIBLE;
    top->special |= POSSIBLE_EN_PASSANT;
    top++;
  }
  return top;
}

Move * b_king_non_cap(Move * top){
  int to;
  BitBoard to_board;
  to_board = king_moves[game.black_king]& ~(game.b_pieces|game.w_pieces);
  while (to_board) {
    to=get_first_lsb(to_board);
    to_board^=mask[to];
    top->from=game.black_king;
    top->to=to;
    top->to_promote=0;
    top->special = 0;
    top++;
  }
  return top;
}

Move * b_castles(Move * top){
  Move stack[100];
  Move * my_top = &(stack[0]);
  int flag;
  if((game.new_castle & (B_BIG_CASTLE | B_SMALL_CASTLE)) == 0){
    return top;
  }else{
    if(game.new_castle & B_SMALL_CASTLE){
      /* lets check if we can castle right*/
      if(!game.board[5] && !game.board[6] && game.board[12] != w_pawn){
	/* the path is clear, now we must check for attacked squares */
	my_top = all_w_cap(my_top);
	my_top = all_w_non_cap_non_castle(my_top);
	flag = 0;
	while(my_top > &stack[0]){
	  my_top--;
	  if(my_top->to == E8 || my_top->to == F8 || my_top->to == G8){
	    flag = 1;
	    break;
	  }
	}
	if(flag == 0){
	  top->from = CASTLE_FROM;
	  top->to = CASTLE_TO;
	  top->special = B_SMALL_CASTLE;
	  top->to_promote = 0;
	  top++;
	}
      }
    }
    if(game.new_castle & B_BIG_CASTLE){
      /* lets check if we can castle left*/
      if(!game.board[1] && !game.board[2] && !game.board[3] && game.board[12] != w_pawn){
	/* the path is clear */
	/* the path is clear, now we must check for attacked squares */
	my_top = &(stack[0]);
	my_top = all_w_cap(my_top);
	my_top = all_w_non_cap_non_castle(my_top);
	flag = 0;
	while(my_top > &stack[0]){
	  my_top--;
	  if(my_top->to == E8 || my_top->to == D8 || my_top->to == C8){
	    flag = 1;
	    break;
	  }
	}
	if(flag == 0){
	  top->from = CASTLE_FROM;
	  top->to = CASTLE_TO;
	  top->special = B_BIG_CASTLE;
	  top->to_promote = 0;
	  top++;
	}
	
      }
    }
  }
  return top;
}



Move * all_b_non_cap(Move * top){
  top = b_castles(top);
  top = b_knight_non_cap(top);
  top = b_bishop_non_cap(top);
  top = b_rook_non_cap(top);
  top = b_queen_non_cap(top);
  /* this are pseudo legal moves */
  top = b_king_non_cap(top);
  top = b_pawn_non_cap(top);
  return top;
}

Move * all_b_non_cap_non_castle(Move * top){
  top = b_knight_non_cap(top);
  top = b_bishop_non_cap(top);
  top = b_rook_non_cap(top);
  top = b_queen_non_cap(top);
  top = b_pawn_non_cap(top);
  /* this are pseudo legal moves */
  top = b_king_non_cap(top);
  return top;
}


/***************************************************************
  
This is the Evades zone where evasions from checks are generated

****************************************************************/

Move * check_evasions(Move * top,char side){
  BitBoard attackers;
  BitBoard to_board;
  BitBoard attacked;
  BitBoard a;
  int from;
  int n_atta;
  int to;
  if(side == WHITE){
    if(in_check(!game.to_move)){      
      /* check if we can capture opposing king*/
      attackers = gen_attackers(game.black_king) & game.w_pieces;
      from = get_first_lsb(attackers);
      top->from = from;
      top->to = game.black_king;
      to = top->to;
      if(game.board[from] !=w_pawn){
	top->to_promote = 0;
	top->special = CAPTURE;
	top->special |= IRREVERSIBLE;
      }else{
	top->special = IRREVERSIBLE;
	if(ROW(to) == 0){
	  top->to_promote = QUEEN;
	}else{
	  top->to_promote = 0;
	}
	top->special |= CAPTURE;
      }
      top++;
      return top;
    }
    attackers = gen_attackers(game.white_king) & game.b_pieces;
    if(attackers & mask[game.black_king]){
      /* how nice we have an illegal position with 2 kings side by side*/
      top->from=game.white_king;
      top->to=game.black_king;
      top->to_promote=0;
      top->special = CAPTURE;
      top->special |= IRREVERSIBLE;
      top++;
      return top;
    }
    n_atta = pop_count(attackers);
    attacked = gen_attacks(BLACK);

    /* First lets generate the non capture king moves */      
    to_board = (king_moves[game.white_king]& ~(game.b_pieces|game.w_pieces)) & ~attacked;
    while(to_board){
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      /*
	Now we're gonna use a little trick
         The move is only valid if the king was not blocking an attack
	 on the "to" square. That's why we use this strange pinned test
      */
      if(!pinned_to_square(game.white_king,to)){
	top->from=game.white_king;
	top->to=to;
	top->to_promote=0;
	top->special = 0;
	top++;      
      }
    }
    /* generate interpositions */
    if(n_atta == 1){
      to_board = path[get_first_lsb(attackers)][game.white_king] & gen_moves(WHITE);
      while(to_board){
	to = get_first_lsb(to_board);
	to_board^=mask[to];
	a = gen_movers(WHITE,to) & ~mask[game.white_king];
	while(a){
	  from = get_first_lsb(a);
	  a^=mask[from];
	  if(!pinned_to_square(from,game.white_king)){
	    top->from = from;
	    top->to = to;
	    if(game.board[from] !=w_pawn){
	      top->to_promote = 0;
	      top->special = 0;
	    }else{
	      /* 
		 Fortunately an interposition can never be an en_passant (Blacky said it)
		 so we don't have to take care with that 
	      */
	      top->special = IRREVERSIBLE;
	      if(ROW(to) == 0){
		top->to_promote = QUEEN;
	      }else{
		top->to_promote = 0;
	      }
	      if(ROW(to)+2 == ROW(from)){
		/*we got a 2 square pawn move */
		top->special |= POSSIBLE_EN_PASSANT;
	      }
	    }
	    top++;
	  }
	}
      }
      /* generate non-king and non-en passant captures */
      a = gen_attackers(get_first_lsb(attackers)) & game.w_pieces & ~mask[game.white_king];
      while(a){
	from = get_first_lsb(a);
	a^=mask[from];
	top->from = from;
	top->to = get_first_lsb(attackers);
	to = top->to;
	if(game.board[from] !=w_pawn){
	  top->to_promote = 0;
	  top->special = CAPTURE;
	  top->special |= IRREVERSIBLE;
	}else{
	  top->special = IRREVERSIBLE;
	  if(ROW(to) == 0){
	    top->to_promote = QUEEN;
	  }else{
	    top->to_promote = 0;
	  }       
	  top->special |= CAPTURE;
	}
	top++;
      }
      /* generate en passant captures */
      if(game.en_passant && game.en_passant == get_first_lsb(attackers)-8){
	a = gen_attackers(game.en_passant) & WPawns;
	while(a){
	  from = get_first_lsb(a);
	  a^=mask[from];
	  top->from = from;
	  top->to = game.en_passant;
	  to = top->to;
	  top->special = IRREVERSIBLE;
	  /* there are no enpassant promotions*/
	  /*	  if(ROW(to) == 0){
		  top->to_promote = QUEEN;
		  }else{
		  top->to_promote = 0;
		  } 
	  */      
	  top->to_promote = 0;

	  top->special |= CAPTURE;
	  top->special |= EN_PASSANT;
	  top++;	
	}
      }
    }
    /* now generate capture king moves */
    to_board = (king_moves[game.white_king] & game.b_pieces) & ~attacked;
    while(to_board){
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      /*
	Now we're gonna use a little trick
	The move is only valid if the king was not blocking an attack
	on the "to" square. That's why we use this strange pinned test
      */
      if(!pinned_to_square(game.white_king,to)){
	top->from=game.white_king;
	top->to=to;
	top->to_promote=0;
	top->special = CAPTURE;
	top->special |= IRREVERSIBLE;
	top++;      
      }
    }
  }else{
    if(in_check(!game.to_move)){      
      /* check if we can capture opposing king*/
      attackers = gen_attackers(game.white_king) & game.b_pieces;
      from = get_first_lsb(attackers);
      top->from = from;
      top->to = game.white_king;
      to = top->to;
      if(game.board[from] !=b_pawn){
	top->to_promote = 0;
	top->special = CAPTURE;
	top->special |= IRREVERSIBLE;
      }else{
	top->special = IRREVERSIBLE;
	if((ROW(to)) == 7){
	  top->to_promote = QUEEN;
	}else{
	  top->to_promote = 0;
	}
	top->special |= CAPTURE;
      }
      top++;
      return top;
    }
    attackers = gen_attackers(game.black_king) & game.w_pieces;
    if(attackers & mask[game.white_king]){
      /* how nice we have an illegal position with 2 kings side by side*/
      top->from=game.black_king;
      top->to=game.white_king;
      top->to_promote=0;
      top->special = CAPTURE;
      top->special |= IRREVERSIBLE;
      top++;
      return top;
    }
    n_atta = pop_count(attackers);
    attacked = gen_attacks(WHITE);

    /* First lets generate the non capture king moves */      
    to_board = (king_moves[game.black_king]& ~(game.b_pieces|game.w_pieces)) & ~attacked;
    while(to_board){
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      /*
	Now we're gonna use a little trick
         The move is only valid if the king was not blocking an attack
	 on the "to" square. That's why we use this strange pinned test
      */
      if(!pinned_to_square(game.black_king,to)){
	top->from=game.black_king;
	top->to=to;
	top->to_promote=0;
	top->special = 0;
	top++;      
      }
    }
    /* generate interpositions */
    if(n_atta == 1){
      to_board = path[get_first_lsb(attackers)][game.black_king] & gen_moves(BLACK);
      while(to_board){
	to = get_first_lsb(to_board);
	to_board^=mask[to];
	a = gen_movers(BLACK,to) & ~mask[game.black_king];
	while(a){
	  from = get_first_lsb(a);
	  a^=mask[from];
	  if(!pinned_to_square(from,game.black_king)){
	    top->from = from;
	    top->to = to;
	    if(game.board[from] !=b_pawn){
	      top->to_promote = 0;
	      top->special = 0;
	    }else{
	      /* 
		 Fortunately an interposition can never be an en_passant (Blacky said it)
		 so we don't have to take care with that 
	      */
	      top->special = IRREVERSIBLE;
	      if((ROW(to)) == 7){
		top->to_promote = QUEEN;
	      }else{
		top->to_promote = 0;
	      }
	      if(ROW(to)-2 == ROW(from)){
		/*we got a 2 square pawn move */
		top->special |= POSSIBLE_EN_PASSANT;
	      }
	    }
	    top++;
	  }
	}
      }
      /* generate non king captures */
      a = gen_attackers(get_first_lsb(attackers)) & game.b_pieces & ~mask[game.black_king];
      while(a){
	from = get_first_lsb(a);
	a^=mask[from];
	top->from = from;
	top->to = get_first_lsb(attackers);
	to = top->to;
	if(game.board[from] !=b_pawn){
	  top->to_promote = 0;
	  top->special = CAPTURE;
	  top->special |= IRREVERSIBLE;
	}else{
	  top->special = IRREVERSIBLE;
	  if((ROW(to)) == 7){
	    top->to_promote = QUEEN;
	  }else{
	    top->to_promote = 0;
	  }
	  top->special |= CAPTURE;
	}
	top++;
      }      
    }
    /* generate en passant captures */
    if(game.en_passant && game.en_passant == get_first_lsb(attackers)+8){
      a = gen_attackers(game.en_passant) & BPawns;
	while(a){
	  from = get_first_lsb(a);
	  a^=mask[from];
	  top->from = from;
	  top->to = game.en_passant;
	  to = top->to;
	  top->special = IRREVERSIBLE;
	  top->to_promote = 0;
	  top->special |= CAPTURE;
	  top->special |= EN_PASSANT;
	  top++;	
	}
      }
    /* now generate capture king moves */
    to_board = (king_moves[game.black_king] & game.w_pieces) & ~attacked;
    while(to_board){
      to=get_first_lsb(to_board);
      to_board^=mask[to];
      /*
	Now we're gonna use a little trick
         The move is only valid if the king was not blocking an attack
	 on the "to" square. That's why we use this strange pinned test
      */
      if(!pinned_to_square(game.black_king,to)){
	top->from=game.black_king;
	top->to=to;
	top->to_promote=0;
	top->special = CAPTURE;
	top->special |= IRREVERSIBLE;
	top++;      
      }
    }
  }
  return top;
}

Move * valid_moves(Move * top){
  Move stack[MAX_POSSIBLE_MOVES];
  Move * my_top = &stack[0];
  Move * floor = my_top;
  if(game.to_move == WHITE){
    my_top = all_w_cap(my_top);
  }else{
    my_top = all_b_cap(my_top);
  }
  if(game.to_move == WHITE){
    my_top = all_w_non_cap(my_top);
  }else{
    my_top = all_b_non_cap(my_top);
  }
  while(my_top > floor){
    my_top--;
    make_move(*my_top);
    /* check if player who moved has his king in check */
    /* if so this is an invalid positoin */
    if(!in_check(!game.to_move)){
      *top = *my_top;
      top++;
    }
    undo_move(*my_top);
  }
  return top;
}
      


int perft(int depth){
  int i = 0;  
  Move stack[MAX_POSSIBLE_MOVES];
  Move * top = &stack[0];
  Move * floor = top;
  if(game.white_king == -1 || game.black_king == -1){
    error("Illegal move executed!\n");
  }
  if(in_check(!game.to_move)){
    return 0;
  }
  if(depth == 0){
    return 1;
  }
  if(game.to_move == WHITE){
    top = all_w_cap_non_promotions(top);
    top = all_w_non_cap(top);
  }else{
    top = all_b_cap_non_promotions(top);
    top = all_b_non_cap(top);
  }
  while(top>floor){
    top--;
    make_move(*top);
    i += perft(depth-1);
    undo_move(*top);
  }
  return i;
}
