/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#ifndef _GEN_MOVES_H_
#define _GEN_MOVES_H_

#define MAX_POSSIBLE_MOVES 300

/* this 2 values must compose a completly impossible move because
 of the val_table */
#define CASTLE_FROM 0
#define CASTLE_TO 62

/* specials bit list */
#define W_BIG_CASTLE 0x01
#define W_SMALL_CASTLE 0x02
#define W_CASTLE 0x07
#define EN_PASSANT 0x04
#define B_BIG_CASTLE 0x08
#define B_SMALL_CASTLE 0x10
#define B_CASTLE 0x38
#define CAPTURE 0x20
#define IRREVERSIBLE 0x40
#define POSSIBLE_EN_PASSANT 0x80
#define W_CASTLED 0x04
#define B_CASTLED 0x20

typedef struct{
  unsigned char from;
  unsigned char to;
  unsigned char to_promote;
  unsigned char special;
} Move;


Move * all_w_cap( Move * top);
Move * all_w_cap_non_promotions( Move * top);
Move * w_knight_cap(Move * top);
Move * w_bishop_cap(Move * top);
Move * w_rook_cap(Move * top);
Move * w_queen_cap(Move * top);
Move * w_pawn_cap(Move * top);
Move * w_pawn_promotions(Move * top);

Move * all_w_non_cap( Move * top);
Move * w_knight_non_cap(Move * top);
Move * w_bishop_non_cap(Move * top);
Move * w_rook_non_cap(Move * top);
Move * w_queen_non_cap(Move * top);
Move * w_castles(Move * top);
Move * w_pawn_non_cap(Move * top);
Move * w_pawn_non_cap_non_castle(Move * top);

Move * all_b_cap_non_promotions( Move * top);
Move * all_b_cap( Move * top);
Move * b_knight_cap(Move * top);
Move * b_bishop_cap(Move * top);
Move * b_rook_cap(Move * top);
Move * b_queen_cap(Move * top);
Move * b_pawn_cap(Move * top);
Move * b_pawn_promotions(Move * top);

Move * all_b_non_cap( Move * top);
Move * all_b_non_cap_non_castle( Move * top);
Move * b_knight_non_cap(Move * top);
Move * b_bishop_non_cap(Move * top);
Move * b_rook_non_cap(Move * top);
Move * b_queen_non_cap(Move * top);
Move * b_castles(Move * top);
Move * b_pawn_non_cap(Move * top);

Move * check_evasions(Move * top,char side);

/* this is terribly slow and should only be used in the root */
Move * valid_moves(Move * top);
int perft(int depth);
#endif
