/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include "bitboard.h"
#include <gen_moves.h>
#include "search.h"
#include "h_heuristic.h"
#include <math_util.h>

/*int history_heuristics[color][from][to]*/
int history_heuristics[64][64];

void add_move_to_hh(Move  top,int depth){
  if(stop_search != 1){
    history_heuristics[(int)top.from][(int)top.to]+=depth*depth;
  }
}

int score_move(Move * top){
  return  history_heuristics[(int)top->from][(int)top->to];
}

void init_history_heuristics(){
  int j,k;
  for(j = 0;j<64;j++){
    for(k = 0;k<64;k++){
      history_heuristics[j][k] = 0;
    }
  }
}

void history_sort_moves(Move * m, int n){
  int i, j;
  Move temp;
  /*we'll only order the first 3 moves, as crafty, the rest is irrelevant*/
  /* good old bubble sort */
  for (i = (n - 1); i >= max(0,(n-3)); i--)
  {
    for (j = 1; j <= i; j++)
    {
      if(score_move(&(m[j-1])) > score_move(&(m[j])))
	{
	  temp = m[j-1];
	  m[j-1] = m[j];
	  m[j] = temp;
	}
    }
  }
}

void shift_history(){
  int i,j;
  for(i = 0;i<64;i++){
    for(j = 0;j<64;j++){
      history_heuristics[i][j] = history_heuristics[i][j] >> HH_SHIFT;
    }
  }
}
