/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include <gen_moves.h>
#include "bitboard.h"
#include "search.h"
#include "util.h"
#include "tt.h"
#include "string.h"
#include "validate.h"

int passed_scale = 100;

signed char kval_wn[64] = {
  -50, -40, -30, -20, -20, -30, -40, -50,
  -30, -20, -10,   0,   0, -10, -20, -30,
  -30, -10,   0,  10,  10,   0, -10, -30,
  -30, -10,  20,  30,  30,  20, -10, -30,
  -30, -10,  30,  40,  40,  30, -10, -30,
  -30, -10,  30,  40,  40,  30, -10, -30,
  -30, -30,   0,   0,   0,   0, -30, -30,
  -50, -30, -30, -30, -30, -30, -30, -50
};
signed char kval_wk[64] = {
  -90, -70, -50, -40, -30, -20, -20, -30,
  -70, -50, -30, -20, -10, -10, -10, -10,
  -70, -50, -30, -10,  20,  20,  20,   0,
  -70, -50, -30, -10,  30,  30,  30,   0,
  -70, -50, -30, -10,  30,  30,  30,   0,
  -70, -50, -30, -10,  30,  30,  30,   0,
  -70, -50, -30, -30,   0,   0,   0, -10,
  -90, -70, -50, -30, -30, -30, -30, -30
};

signed char kval_wq[64] = {
  -30, -20, -20, -30, -40, -50, -70, -90,
  -10, -10, -10, -10, -20, -30, -50, -70,
    0,  20,  20,  20, -10, -30, -50, -70,
    0,  30,  30,  30, -10, -30, -50, -70,
    0,  30,  30,  30, -10, -30, -50, -70,
    0,  30,  30,  30, -10, -30, -50, -70,
  -10,   0,   0,   0, -20, -30, -50, -70,
  -30, -30, -30, -30, -30, -50, -70, -90
};



const char outside_passed[128] = {
  96, 48, 48, 48, 45, 42, 40, 40,
  36, 36, 32, 32, 28, 28, 24, 24,
  20, 20, 16, 16, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12,
  12, 12, 12, 12, 12, 12, 12, 12
};


const int even_diag_shift[64] = { 
  28,21,15,10,06,03,01,00,
  36,28,21,15,10,06,03,01,
  43,36,28,21,15,10,06,03,
  49,43,36,28,21,15,10,06,
  54,49,43,36,28,21,15,10,
  58,54,49,43,36,28,21,15,
  61,58,54,49,43,36,28,21,
  63,61,58,54,49,43,36,28
};
  

const int odd_diag_shift[64] = {
  00,01,03,06,10,15,21,28,
  01,03,06,10,15,21,28,36,
  03,06,10,15,21,28,36,43,
  06,10,15,21,28,36,43,49,
  10,15,21,28,36,43,49,54,
  15,21,28,36,43,49,54,58,
  21,28,36,43,49,54,58,61,
  28,36,43,49,54,58,61,63
};

const int r_0_shift[64] = {
  0,0,0,0,0,0,0,0,
  8,8,8,8,8,8,8,8,
  16,16,16,16,16,16,16,16,
  24,24,24,24,24,24,24,24,
  32,32,32,32,32,32,32,32,
  40,40,40,40,40,40,40,40,
  48,48,48,48,48,48,48,48,
  56,56,56,56,56,56,56,56
};

const int r_90_shift[64] = {
  0,8,16,24,32,40,48,56,
  0,8,16,24,32,40,48,56,
  0,8,16,24,32,40,48,56,
  0,8,16,24,32,40,48,56,
  0,8,16,24,32,40,48,56,
  0,8,16,24,32,40,48,56,
  0,8,16,24,32,40,48,56,
  0,8,16,24,32,40,48,56
};


const int len_even_diag[64] = { 
  8,7,6,5,4,3,2,1,
  7,8,7,6,5,4,3,2,
  6,7,8,7,6,5,4,3,
  5,6,7,8,7,6,5,4,
  4,5,6,7,8,7,6,5,
  3,4,5,6,7,8,7,6,
  2,3,4,5,6,7,8,7,
  1,2,3,4,5,6,7,8 
};

const int len_odd_diag[64] = { 
  1,2,3,4,5,6,7,8,
  2,3,4,5,6,7,8,7,
  3,4,5,6,7,8,7,6,
  4,5,6,7,8,7,6,5,
  5,6,7,8,7,6,5,4,
  6,7,8,7,6,5,4,3,
  7,8,7,6,5,4,3,2,
  8,7,6,5,4,3,2,1 
};


const int r_90_map[64] = {
 7,15,23,31,39,47,55,63,
 6,14,22,30,38,46,54,62,
 5,13,21,29,37,45,53,61,
 4,12,20,28,36,44,52,60,
 3,11,19,27,35,43,51,59,
 2,10,18,26,34,42,50,58,
 1, 9,17,25,33,41,49,57,
 0, 8,16,24,32,40,48,56 };



const int even_diag_map[64] = { 
  28,21,15,10, 6, 3, 1, 0,
  36,29,22,16,11, 7, 4, 2,
  43,37,30,23,17,12, 8, 5,
  49,44,38,31,24,18,13, 9,
  54,50,45,39,32,25,19,14,
  58,55,51,46,40,33,26,20,
  61,59,56,52,47,41,34,27,
  63,62,60,57,53,48,42,35 
};


const int odd_diag_map[64] = { 
  0 , 1, 3, 6,10,15,21,28,
  2 , 4, 7,11,16,22,29,36,
  5 , 8,12,17,23,30,37,43,
  9 ,13,18,24,31,38,44,49,
  14,19,25,32,39,45,50,54,
  20,26,33,40,46,51,55,58,
  27,34,41,47,52,56,59,61,
  35,42,48,53,57,60,62,63
};

const signed char white_outpost[64] = { 0, 0, 0, 0, 0, 0, 0, 0,
			                0, 0, 0, 0, 0, 0, 0, 0,
				        0, 0, 0, 0, 0, 0, 0, 0,
				        0, 0, 0, 6, 6, 0, 0, 0,
				        0, 0,12,15,15,12, 0, 0,
				        0, 0,12,18,18,12, 0, 0,
				        0, 0, 6,12,12, 6, 0, 0,
				        0, 0, 0, 0, 0, 0, 0, 0 };




/* They aren't const because they take to much work to initialize by hand */
BitBoard knight_moves[64];
BitBoard king_moves[64];
BitBoard rook_rank_moves[64][256];
BitBoard rook_file_moves[64][256];
BitBoard bishop_odd_moves[64][256];
BitBoard bishop_even_moves[64][256];

/* same as in crafty */
BitBoard wtm_random[2];




void init_w_pawns(BitBoard * p){
  int i;
  BitBoard a = 1;
  a = a << 55;
  *p = 0;

  for(i = 0;i<8;i++){
    *p |= a;
    a = a >> 1;
  }    
}
     
void init_b_pawns(BitBoard * p){
  int i;
  BitBoard a = 1;
  a = a << 8;
  *p = 0;
  for(i = 0;i<8;i++){
    *p |= a;
    a = a << 1;
  }    
}

void init_b_knights(BitBoard * p){
  BitBoard a = 1;
  *p = 0;
  *p |= a << 1;
  *p |= a << 6;
}

void init_w_knights(BitBoard * p){
  BitBoard a = 1;
  a = a << 63;
  *p = 0;
  *p |= a >> 1;
  *p |= a >> 6;
}

void init_b_bishops(BitBoard * p){
  BitBoard a = 1;
  *p = 0;
  *p |= a << 2;
  *p |= a << 5;
}

void init_w_bishops(BitBoard * p){
  BitBoard a = 1;
  a = a << 63;
  *p = 0;
  *p |= a >> 2;
  *p |= a >> 5;
}

void init_b_rooks(BitBoard * p){
  BitBoard a = 1;
  *p = 0;
  *p |= a;
  *p |= a << 7;
}
  
void init_w_rooks(BitBoard * p){
  BitBoard a = 1;
  a = a << 63;
  *p = 0;
  *p |= a;
  *p |= a >> 7;
}    

void init_b_queens(BitBoard *p){
  BitBoard a = 1;
  *p = 0;
  *p |= a << 3;
}    

void init_w_queens(BitBoard *p){
  BitBoard a = 1;
  a = a << 63;
  *p = 0;
  *p |= a >> 4;
}    

void init_queen_bishops(BitBoard * p){
  BitBoard a = 1;
  *p = 0;
  *p |= a << 2;
  *p |= a << (63-5);
}

void init_queen_rooks(BitBoard * p){
  BitBoard a = 1;
  *p = 0;
  *p |= a << 0;
  *p |= a << 56;
}

void init_all_r_90(BitBoard * p){
  int i;
  BitBoard a = 1;
  *p = 0;
  for(i = 0;i<64;i++){
    if(i<16 || i>47){
      *p |= a << r_90_map[i] ;
    }
  }
}

void init_all_even_diag(BitBoard * p){
  int i;
  BitBoard a = 1;
  *p = 0;
  for(i = 0;i<64;i++){
    if(i<16 || i>47){
      *p |= a << even_diag_map[i] ;
    }
  }
} 

void init_all_odd_diag(BitBoard * p){
  int i;
  BitBoard a = 1;
  *p = 0;
  for(i = 0;i<64;i++){
    if(i<16 || i>47){
      *p |= a << odd_diag_map[i];
    }
  }
} 


  


