/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/
#include <bitboard.h>
#include <init.h>
#include <chess_const.h>
#include <gen_moves.h>
#include <init_moves.h>


void init_king_moves(){
  int p,i,j,rank,file;
  BitBoard a;
  for(p = 0;p<64;p++){ 
    king_moves[p] = 0;
    for(i = -1;i<2;i++){
      for(j = -1;j<2;j++){
	rank = RANK(p);
	file = COL(p);
	if((i || j) && 
	   rank+i < 8 && rank+i >= 0 &&
	   file+j <8  && file+j >= 0){
	  a = 1;
	  king_moves[p] |= a<< (p+j-i*8);
	}
      }
    }
  }      
}

void init_knight_moves(){
  int i;
  int file,rank;
  int target;
  BitBoard a;
  for(i = 0;i<64;i++){
    knight_moves[i] = 0;
    file = COL(i);
    rank = RANK(i);
    if(file + 1 < 8){
      if(rank +2 < 8){
	target = i-2*8+1;

	a = 1;
	a = a << target;
	knight_moves[i] |= a;
      }
      if(rank -2 >= 0){
	target = i+1+2*8;
	a = 1;
	a = a << target;
	knight_moves[i] |= a;
      }	
      if(file +2 < 8){
	if(rank +1 < 8){
	  target = i+2-1*8;
	  a = 1;
	  a = a << target;
	  knight_moves[i] |= a;
	}
	if(rank -1 >= 0){
	  target = i+2+1*8;
	  a = 1;
	  a = a << target;
	  knight_moves[i] |= a;
	}
      }
    }
	
    if(file - 1 >= 0){
      if(rank +2 < 8){
	target = i-1-2*8;
	a = 1;
	a = a << target;
	knight_moves[i] |= a;
      }
      if(rank -2 >= 0){
	target = i-1+2*8;
	a = 1;
	a = a << target;
	knight_moves[i] |= a;
      }	
      if(file -2 >= 0){
	if(rank +1 < 8){
	  target = i-2-1*8;
	  a = 1;
	  a = a << target;
	  knight_moves[i] |= a;
	}
	if(rank -1 >= 0){
	  target = i-2+1*8;
	  a = 1;
	  a = a << target;
	  knight_moves[i] |= a;
	}
      }
    }
  }
}


void init_rook_file_moves(){
  int i,k,j;
  BitBoard a = 0;
  for(i =0;i<64;i++){
    for(j = 0;j<256;j++){
      /*most significant bit will be x8*/
      rook_file_moves[i][j] = 0;
      /* first lets move up */
      for(k = 1;(i-8*k) >= 0;k++){
	a = 1;
	rook_file_moves[i][j] |= a << (i-k*8);
	if((1 << ( RANK((i-k*8)) ) ) & j){
	  break;
	}
      }
      /* and now down */
      for(k = 1;(i+8*k) < 64;k++){
	a = 1;
	rook_file_moves[i][j] |= a << (i+k*8);
	if((1 <<(RANK((i+k*8)))) & j){
	  break;
	}
      }
    }
  }
}


void init_rook_rank_moves(){
  unsigned int i,k,j;
  BitBoard a = 0;
  for(i =0;i<64;i++){
    for(j = 0;j<256;j++){
      /*most significant bit will be Hx*/
      rook_rank_moves[i][j] = 0;
      /* first lets for left */
      for(k = 1;((i-k)%8) != 7 ;k++){
	a = 1;
	rook_rank_moves[i][j] |= a << (i-k);
	if((1 << (((i-k)%8))) & j){
	  break;
	}
      }
      /* and now right */
      for(k = 1;(i+k)%8;k++){
	a = 1;
	rook_rank_moves[i][j] |= a << (i+k);
	if((1 << (((i+k)%8))) & j){
	  break;
	}
      }
    }
  }
}


void init_bishop_even_moves(){
  int i,k,j;
  int rank,file;
  int diag_pos;
  int diag_len;
  BitBoard a = 0;
  
  for(i =0;i<64;i++){
    diag_len = len_even_diag[i] -1;
    file = COL(i);
    rank = RANK(i);
    diag_pos = file;
    if(7-rank<diag_pos){
      diag_pos = 7-rank;
    }
    for(j = 0;j<256;j++){
      /*most significant bit will be H8*/
      bishop_even_moves[i][j] = 0;
      /* first lets for lower right */
      for(k = 1;file + k< 8 && rank -k >= 0;k++){
	a = 1;
	bishop_even_moves[i][j] |= a << (i+k*9);
	/*we got to be careful here because j does not represent
	 the entire line of the rotated bitboard but only the relevant part*/
	/* this means that for example for certain i (say 58) a j of 2 means that
	 b2 is occupied, NOT g2 */
	if((1 << (diag_pos+k)) & j){
	  break;
	}
      }
      /* and now upper left */
      for(k = 1;file - k>= 0 && rank +k < 8;k++){
	a = 1;
	bishop_even_moves[i][j] |= a << (i-k*9);
	if((1 << (diag_pos-k)) & j){
	  break;
	}
      }
    }
  }

}

void init_bishop_odd_moves(){
  int i,k,j;
  int rank,file;
  int diag_pos;
  int diag_len;
  BitBoard a = 0;
  
  for(i =0;i<64;i++){
    diag_len = len_odd_diag[i] -1;
    file = COL(i);
    rank = RANK(i);
    if(7-file<7-rank){
      diag_pos = 7-file;
    }else{
      diag_pos = 7-rank;
    }
    for(j = 0;j<256;j++){
      /*most significant bit will be A1*/
      bishop_odd_moves[i][j] = 0;
      /* first lets go for lower left */
      for(k = 1;file - k>= 0 && rank -k >= 0;k++){
	a = 1;
	bishop_odd_moves[i][j] |= a << (i+k*7);
	/*we got to be careful here because j does not represent
	 the entire line of the rotated bitboard but only the relevant part*/
	/* this means that for example for certain i (say 62) a j of 2 means that
	 h2 is occupied, NOT b2 */
	if((1 << (diag_pos+k)) & j){
	  break;
	}
      }
      /* and now upper right */
      for(k = 1;file + k< 8 && rank +k < 8;k++){
	a = 1;
	bishop_odd_moves[i][j] |= a << (i-k*7);
	if((1 << (diag_pos-k)) & j){
	  break;
	}
      }
    }
  }
}
