/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include <ctype.h>
#include <string.h>
#include <time.h>
#include <gen_moves.h>
#include <chess_const.h>
#include "bitboard.h"
#include "search.h"
#include "util.h"
#include "commands.h"
#include "communicate.h"
#include "check.h"
#include "make_moves.h"
#include "interface.h"
#include <init.h>
#include <controls.h>
#include <communicate.h>


int alg2cart(char a){
  if(isdigit(a)){
    return (int)a-(int)'1';
  }else{
    return (int)a-(int)'a';
  }
  return -1;
}


char cart2alg(int d){
  return (char)((int)'a'+d);
}

int piece_to_num(char piece){
  if(piece == 'K'){
    return KING;
  }else if(piece == 'Q'){
    return QUEEN;
  }else if(piece == 'B'){
    return BISHOP;
  }else if(piece == 'N'){
    return KNIGHT;
  }else if(piece == 'R'){
    return ROOK;
  }else if(piece == 'P'){
    return PAWN;
  }else{
    error("Weird piece moving around here!");
  }
  return -1;
}

char num_to_piece(int piece){
  if(piece == KING){
    return 'K';
  }else if(piece == QUEEN){
    return 'Q';
  }else if(piece == BISHOP){
    return 'B';
  }else if(piece == KNIGHT){
    return 'N';
  }else if(piece == ROOK){
    return 'R';
  }else if(piece == PAWN){
    return 'P';
  }else{
    error("Weird piece moving around here!");
  }
  return -1;
}


Move algebraic2move(char * alg, Position * p){
  Move ret;
  Move stack[MAX_POSSIBLE_MOVES];
  Move * top = &(stack[0]);
  Move * floor = &(stack[0]);
  char sx,sy,dx,dy;
  char s_file;
  char s_rank;
  char e_file;
  char e_rank;
  Colour_Piece piece;
  int promotion;
  ret.from = 0;
  ret.to = 0;
  ret.special = 0;
  if(p->to_move == WHITE){
     top = all_w_cap(top);
     top = all_w_non_cap(top);
     if(strstr(alg,"O-O-O") || strstr(alg,"o-o-o")){
       ret.from = CASTLE_FROM;
       ret.to = CASTLE_TO;
       ret.special |= W_BIG_CASTLE;
       ret.to_promote = 0;
     }else if(strstr(alg,"O-O") || strstr(alg,"o-o")){
       ret.from = CASTLE_FROM;
       ret.to = CASTLE_TO;
       ret.special |= W_SMALL_CASTLE;
       ret.to_promote = 0;
     }else if(isupper(alg[0])){
       /*We have a piece*/
       piece = piece_to_num(alg[0]);
       if(isalpha(alg[1]) && alg[1] != 'x'){
	 dx = alg[1];
	 if(isalpha(alg[2]) && alg[2] != 'x'){
	   /* We have something like Nbc2 */
	   sx = dx;
	   dx = alg[2];
	   dy = alg[3];
	   while(top > floor){
	     top--;
	     e_file = top->to%8;
	     e_rank = 7-top->to/8;
	     s_file = top->from%8;
	     if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) && s_file == alg2cart(sx)
		&& p->board[top->from] == piece){
	       make_move(*top);
	       if(!in_check(!game.to_move)){
		 /*We got our men*/
		 ret = *top;
		 undo_move(*top);
		 break;
	       }else{
		 undo_move(*top);
	       }
	     }
	   }
	 }else if(alg[2] == 'x'){
	   /* We have something like Naxb2 */
	   sx = dx;
	   dx = alg[3];
	   dy = alg[4];
	   while(top > floor){
	     top--;
	     e_file = top->to%8;
	     e_rank = 7-top->to/8;
	     s_file = top->from%8;
	     if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) && s_file == alg2cart(sx)
		&& p->board[top->from] == piece){
	       make_move(*top);
	       if(!in_check(!game.to_move)){
		 /*We got our men*/
		 ret = *top;
		 undo_move(*top);
		 break;
	       }else{
		 undo_move(*top);
	       }
	     }
	   }
	 } else if(isdigit(alg[2])){
	   /* We have something like Na3*/
	   dy = alg[2];
	   while(top > floor){
	     top--;
	     e_file = top->to%8;
	     e_rank = 7-top->to/8;
	     s_file = top->from%8;
	     if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) 
		&& p->board[top->from] == piece){
	       make_move(*top);
	       if(!in_check(!game.to_move)){
		 /*We got our men*/
		 ret = *top;
		 undo_move(*top);
		 break;
	       }else{
		 undo_move(*top);
	       }
	     }
	   }
	 } else {
	   error("Shouldn't get here!");
	   fprintf(stderr,"alg - %s\n",alg);
	 }
       } else if(isdigit(alg[1])){
	 sy = alg[1];
	 if(alg[2] == 'x'){	  
	   /* We have N4xd2*/
	   dx = alg[3];
	   dy = alg[4];
	 } else {
	   /* We have N4d2*/
	   dx = alg[2];
	   dy = alg[3];
	 }
	 while(top > floor){
	   top--;
	   e_file = top->to%8;
	   e_rank = 7-top->to/8;
	   s_file = top->from%8;
	   s_rank = 7-top->from/8;
	   if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) 
	      && s_rank == alg2cart(sy) && p->board[top->from] == piece){
	     make_move(*top);
	     if(!in_check(!game.to_move)){
	       /*We got our men*/
	       ret = *top;
	       undo_move(*top);
	       break;
	     }else{
	       undo_move(*top);
	     }
	   }
	 }
       }else if(alg[1] == 'x'){
	 /* We have something nice like Nxc3 */
	 dx = alg[2];
	 dy = alg[3];
	 while(top > floor){
	   top--;
	   e_file = top->to%8;
	   e_rank = 7-top->to/8;
	   s_file = top->from%8;
	   if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) 
	      && p->board[top->from] == piece){
	     make_move(*top);
	     if(!in_check(!game.to_move)){
	       /*We got our men*/
	       ret = *top;
	       undo_move(*top);
	       break;
	     }else{
	       undo_move(*top);
	     }
	   }
	 }
       } else {
	 /* This time we really screwed up */
	 error("Debugging time my friend!");
	 fprintf(stderr,"alg - %s\n",alg);
       }	
     }else{
       /* We have a pawn move */
       /* We only have 2 choices: axb2 a3*/
       /* Special care about the En Passant is needed */
       piece = w_pawn;
       if(isdigit(alg[1])){
	 dx = alg[0];
	 dy = alg[1];
	 promotion = 0;
	 if(alg[2] == '='){
	   promotion = piece_to_num(alg[3]);
	 }
	 while(top > floor){
	   top--;
	   e_file = top->to%8;
	   e_rank = 7-top->to/8;
	   s_file = top->from%8;
	   if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) 
	      && p->board[top->from] == piece && (!promotion || top->to_promote == promotion)){
	     make_move(*top);
	     if(!in_check(!game.to_move)){
	       /*We got our men*/
	       ret = *top;
	       undo_move(*top);
	       break;
	     }else{
	       undo_move(*top);
	     }
	   }
	 }
       }else{
	 sx = alg[0];
	 dx = alg[2];
	 dy = alg[3];
	 promotion = 0;
	 if(alg[4] == '='){
	   promotion = piece_to_num(alg[5]);
	 }
	 while(top > floor){
	   top--;
	   e_file = top->to%8;
	   e_rank = 7-top->to/8;
	   s_file = top->from%8;
	   s_rank = 7-top->from/8;
	   if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) 
	      && s_file == alg2cart(sx) && p->board[top->from] == piece&&
	      (!promotion || top->to_promote == promotion)){
	     make_move(*top);
	     if(!in_check(!game.to_move)){
	       /*We got our men*/
	       ret = *top;
	       undo_move(*top);
	       break;
	     }else{
	       undo_move(*top);
	     }
	   }
	 }
       }
     }
  }else{
    /* Black Move*/
     top = all_b_cap(top);
     top = all_b_non_cap(top);
     if(strstr(alg,"O-O-O") || strstr(alg,"o-o-o")){
       ret.from = CASTLE_FROM;
       ret.to = CASTLE_TO;
       ret.special |= B_BIG_CASTLE;
       ret.to_promote = 0;
     }else if(strstr(alg,"O-O") || strstr(alg,"o-o")){
       ret.from = CASTLE_FROM;
       ret.to = CASTLE_TO;
       ret.special |= B_SMALL_CASTLE;
       ret.to_promote = 0;
     }else if(isupper(alg[0])){
       /*We have a piece*/
       piece = -piece_to_num(alg[0]);
       if(isalpha(alg[1]) && alg[1] != 'x'){
	 dx = alg[1];
	 if(isalpha(alg[2]) && alg[2] != 'x'){
	   /* We have something like Nbc2 */
	   sx = dx;
	   dx = alg[2];
	   dy = alg[3];
	   while(top > floor){
	     top--;
	     e_file = top->to%8;
	     e_rank = 7-top->to/8;
	     s_file = top->from%8;
	     if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) && s_file == alg2cart(sx)
		&& p->board[top->from] == piece){
	       make_move(*top);
	       if(!in_check(!game.to_move)){
		 /*We got our men*/
		 ret = *top;
		 undo_move(*top);
		 break;
	       }else{
		 undo_move(*top);
	       }	       
	     }
	   }
	 }else if(alg[2] == 'x'){
	   /* We have something like Naxb2 */
	   sx = dx;
	   dx = alg[3];
	   dy = alg[4];
	   while(top > floor){
	     top--;
	     e_file = top->to%8;
	     e_rank = 7-top->to/8;
	     s_file = top->from%8;
	     if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) && s_file == alg2cart(sx)
		&& p->board[top->from] == piece){
	       make_move(*top);
	       if(!in_check(!game.to_move)){
		 /*We got our men*/
		 ret = *top;
		 undo_move(*top);
		 break;
	       }else{
		 undo_move(*top);
	       }
	     }
	   }
	 } else if(isdigit(alg[2])){
	   /* We have something like Na3*/
	   dy = alg[2];
	   while(top > floor){
	     top--;
	     e_file = top->to%8;
	     e_rank = 7-top->to/8;
	     s_file = top->from%8;
	     if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) 
		&& p->board[top->from] == piece){
	       make_move(*top);
	       if(!in_check(!game.to_move)){
		 /*We got our men*/
		 ret = *top;
		 undo_move(*top);
		 break;
	       }else{
		 undo_move(*top);
	       }
	     }
	   }
	 } else {
	   error("Shouldn't get here!");
	   fprintf(stderr,"alg - %s\n",alg);
	 }
       } else if(isdigit(alg[1])){
	 sy = alg[1];
	 if(alg[2] == 'x'){	  
	   /* We have N4xd2*/
	   dx = alg[3];
	   dy = alg[4];
	 } else {
	   /* We have N4d2*/
	   dx = alg[2];
	   dy = alg[3];
	 }
	 while(top > floor){
	   top--;
	   e_file = top->to%8;
	   e_rank = 7-top->to/8;
	   s_file = top->from%8;
	   s_rank = 7-top->from/8;
	   if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) 
	      && s_rank == alg2cart(sy) && p->board[top->from] == piece){
	     make_move(*top);
	     if(!in_check(!game.to_move)){
	       /*We got our men*/
	       ret = *top;
	       undo_move(*top);
	       break;
	     }else{
	       undo_move(*top);
	     }
	   }
	 }
       }else if(alg[1] == 'x'){
	 /* We have something nice like Nxc3 */
	 dx = alg[2];
	 dy = alg[3];
	 while(top > floor){
	   top--;
	   e_file = top->to%8;
	   e_rank = 7-top->to/8;
	   s_file = top->from%8;
	   if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) 
	      && p->board[top->from] == piece){
	     make_move(*top);
	     if(!in_check(!game.to_move)){
	       /*We got our men*/
	       ret = *top;
	       undo_move(*top);
	       break;
	     }else{
	       undo_move(*top);
	     }
	   }
	 }
       } else {
	 /* This time we really screwed up */
	 error("Debugging time my friend!");
	 fprintf(stderr,"alg - %s\n",alg);
       }	
     }else{
       /* We have a pawn move */
       /* We only have 2 choices: axb2 a3*/
       /* Special care about the En Passant is needed */
       piece = b_pawn;
       if(isdigit(alg[1])){
	 dx = alg[0];
	 dy = alg[1];
	 promotion = 0;
	 if(alg[2] == '='){
	   promotion = piece_to_num(alg[3]);
	 }
	 while(top > floor){
	   top--;
	   e_file = top->to%8;
	   e_rank = 7-top->to/8;
	   s_file = top->from%8;
	   if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) 
	      && p->board[top->from] == piece && (!promotion || top->to_promote == promotion)){
	     make_move(*top);
	     if(!in_check(!game.to_move)){
	       /*We got our men*/
	       ret = *top;
	       undo_move(*top);
	       break;
	     }else{
	       undo_move(*top);
	     }
	   }
	 }
       }else{
	 sx = alg[0];
	 dx = alg[2];
	 dy = alg[3];
	 promotion = 0;
	 if(alg[4] == '='){
	   promotion = piece_to_num(alg[5]);
	 }
	 while(top > floor){
	   top--;
	   e_file = top->to%8;
	   e_rank = 7-top->to/8;
	   s_file = top->from%8;
	   s_rank = 7-top->from/8;
	   if(e_file == alg2cart(dx) && e_rank == alg2cart(dy) 
	      && s_file == alg2cart(sx) && p->board[top->from] == piece &&
	      (!promotion || top->to_promote == promotion)){
	     make_move(*top);
	     if(!in_check(!game.to_move)){
	       /*We got our men*/
	       ret = *top;
	       undo_move(*top);
	       break;
	     }else{
	       undo_move(*top);
	     }
	   }
	 }
       }
     }
  }
  return ret;
}


char * move2algebraic(Move m, Position *p){
  /*check for pawn*/
  Colour_Piece piece;
  Move stack[100];
  Move * top = &(stack[0]);
  Move * floor = &(stack[0]);
  char * ret;
  char buf[100];
  char e_file;
  char e_rank;
  char s_file;
  ret = malloc(sizeof(char)*10);
  ret[0] = 0;
  /*first check for castle*/
  if((m.special & W_BIG_CASTLE) || (m.special & B_BIG_CASTLE) ){
    strcpy(ret,"O-O-O");
    return ret;
  }else if((m.special & W_SMALL_CASTLE) || (m.special & B_SMALL_CASTLE)){
    strcpy(ret,"O-O");
    return ret;
  }
  piece = p->board[m.from];
  if(piece == w_pawn || piece == b_pawn){
    if(p->board[m.to] || m.special & EN_PASSANT){
      s_file = m.from%8;
      e_file = m.to%8;
      e_rank = 7-m.to/8;
      /* We have something to eat */
      sprintf(ret,"%cx%c%d",cart2alg(s_file),cart2alg(e_file),e_rank+1);
    }else{
      e_rank = 7-m.to/8;
      e_file = m.to%8;

      sprintf(ret,"%c%d",cart2alg(e_file),e_rank+1);
    }
  }else{
    if(p->to_move == WHITE){
      /* white to move */
      /* It's a real piece */
      top = all_w_non_cap(top);
      top = all_w_cap(top);
      sprintf(buf,"%c",num_to_piece(piece));
    }else{
      /* black to move */
      /* It's a real piece */
      top = all_b_non_cap(top);
      top = all_b_cap(top);
      sprintf(buf,"%c",num_to_piece(-piece));
    }
    
    strcat(ret,buf);
    /*We have to check if there are more people that can move to the target*/
    while(top > floor){
      top--;
      e_file = top->to%8;
      e_rank = 7-top->to/8;
      s_file = top->from%8;
      if(top->to == m.to && p->board[top->from] == piece && top->from != m.from ){
	if(s_file != m.from%8){
	  sprintf(buf,"%c",cart2alg(m.from%8));
	  strcat(ret,buf);
	}else{
	  sprintf(buf,"%d",7-m.from/8+1);
	  strcat(ret,buf);
	}	   
      }
    }
    if(p->board[m.to]){
      /* We have something to eat */
      sprintf(buf,"x%c%d",cart2alg(m.to%8),7-m.to/8+1);
    }else{
      sprintf(buf,"%c%d",cart2alg(m.to%8),7-m.to/8+1);
    }
    strcat(ret,buf);
  }
  if(m.to_promote){
    sprintf(buf,"=%c",num_to_piece(m.to_promote));
    strcat(ret,buf);
  }
  return ret;
}


Move get_opponent_move(){
  Move m = {0,0,0,0};
  char buffer[1000];
  FILE * log;
  int flag = 1;
  char * input;

  while(flag){
    if(!x_control.silent){
      printf("Input move\n");
    }
    do{
      input = get_input(1);
      if(g_control.pass_move)
	return m;
    }while(!input);
    m = algebraic2move(input,&game);
    if(m.from == m.to){
      if(!x_control.silent){
	printf("!Invalid move!\n\n");
      }else{
	if(x_control.use_xboard)
	  printf("Error (unable to parse move): %s\n",buffer);
	fprintf(stderr,"-%s-\n",input);
	warn("Unable to parse move!");
	log = fopen("error.log","a");
	fprintf(log,"Error (unable to parse move): -%s-\n",buffer);
	fclose(log);
      }
    }else{
      flag = 0;
    }
  }
  free(input);
  return m;
}

    
      
