/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/
#include <bitboard.h>
#include <math_util.h>
#include <gen_moves.h>
#include "search.h"
#include <util.h>

#ifdef _WIN32
 static __inline__ int ffs(int x)
 {
         int r;
 
         __asm__("bsfl %1,%0\n\t"
                 "jnz 1f\n\t"
                 "movl $-1,%0\n"
                 "1:" : "=r" (r) : "rm" (x));
         return r+1;
 }


#endif
int get_first_lsb(BitBoard p){
  /*  return ffsll(p);*/

  /* ffs() is 32 bit on x86, although the same code should work 
     for 64 bit machines, although with an unnecessary if */
    int ret;
  if((ret = ffs((int)p))){
    return ret-1;
  }else{
    p = p >> 32;
    return ffs((int)p)+31;
  }
  return 0;
}

int count_bits(BitBoard p){
  int c; 
  for (c = 0; p; c++)
    {
      p &= p - 1; 
    }
  return c;
}

BitBoard random_64(void){
  /* same as crafty for zobrist key purposes*/
  BitBoard result;
  unsigned int r1, r2;

  r1 = random_32();
  r2 = random_32();
  result = r1 | (BitBoard) r2 << 32;
  return (result);

}

int distance(char from, char to){
  int f_r,f_f,t_r,t_f;
  int rd,fd;
  f_f = from % 8;
  t_f = to%8;
  f_r = from / 8;
  t_r = to / 8;
  rd = t_r-f_r;
  fd = t_f-f_f;
  if(abs(rd) > abs(fd)){
    return rd;
  }else{
    return fd;
  }
  error("you can't get here");
  return -1;
}

int pop_count(BitBoard a) {
  int res=0;

  while(a) {
    res++;
    a &= a - 1;
  }
  return(res);
}

/* Random32 routine from crafty so that i can (hopefully)
   use their book format 
*/

unsigned int random_32(void){
  /*
    random numbers from Mathematica 2.0.
    SeedRandom = 1;
    Table[Random[Integer, {0, 2^32 - 1}]
  */
  static const unsigned long x[55] = {
    1410651636UL, 3012776752UL, 3497475623UL, 2892145026UL, 1571949714UL,
    3253082284UL, 3489895018UL, 387949491UL, 2597396737UL, 1981903553UL,
    3160251843UL, 129444464UL, 1851443344UL, 4156445905UL, 224604922UL,
    1455067070UL, 3953493484UL, 1460937157UL, 2528362617UL, 317430674UL,
    3229354360UL, 117491133UL, 832845075UL, 1961600170UL, 1321557429UL,
    747750121UL, 545747446UL, 810476036UL, 503334515UL, 4088144633UL,
    2824216555UL, 3738252341UL, 3493754131UL, 3672533954UL, 29494241UL,
    1180928407UL, 4213624418UL, 33062851UL, 3221315737UL, 1145213552UL,
    2957984897UL, 4078668503UL, 2262661702UL, 65478801UL, 2527208841UL,
    1960622036UL, 315685891UL, 1196037864UL, 804614524UL, 1421733266UL,
    2017105031UL, 3882325900UL, 810735053UL, 384606609UL, 2393861397UL
  };
  static int init = 1;
  static unsigned long y[55];
  static int j, k;
  unsigned long ul;

  if (init) {
    int i;

    init = 0;
    for (i = 0; i < 55; i++)
      y[i] = x[i];
    j = 24 - 1;
    k = 55 - 1;
  }

  ul = (y[k] += y[j]);
  if (--j < 0)
    j = 55 - 1;
  if (--k < 0)
    k = 55 - 1;
  return ((unsigned int) ul);
}

