/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/
#include <bitboard.h>
#include <init.h>
#include <score.h>
#include <chess_const.h>
#include <gen_moves.h>
#include "search.h"
#include <negamax.h>
#include <tt.h>
#include <util.h>
#include <zobrist.h>
#include <validate.h>
#include <h_heuristic.h>
#include <quiescent.h>
#include <make_moves.h>
#include <check.h>
#include <communicate.h>

extern int cur_depth;

#define W_Pieces_Material (game.white_material-game.white_pawns*PAWN_VALUE)
#define B_Pieces_Material (game.black_material-game.black_pawns*PAWN_VALUE)

#define FUT_MARGIN BISHOP_VALUE+1
#define RAZOR_MARGIN QUEEN_VALUE +1

Move gkillers[MAX_DEPTH*2][2];
int gkillers_count[MAX_DEPTH*2][2];


int negamax(int depth, int alpha, const int beta, int do_null_move,Line * pline, int ply, Tree_Node * parent){
  int lm;  
#ifdef _DEBUG
  Move debug_stack[MAX_POSSIBLE_MOVES];
  Move * debug_top = &debug_stack[0];
  Move * debug_floor = debug_top;
  char debug_flag = 0;
#endif
  Tree_Node n;
  Move stack[MAX_POSSIBLE_MOVES];
  Move * floor;
  Move m = {0,0,0,0};
  int extensions = 0;
  int i;
  /*  int stale_score;*/
  char color;
  unsigned char en_passant;
  Line line;
  Line dummy_line;
  BitBoard tt_key = game.zob_key &n_bit_mask[TRANS_SZ];
  int init_alpha = alpha;

  int is_in_check = 0;

  /* this stuff is going to be used to stop repeting searches due to TT*/
  Move already_search = {0,0,0,0};

  int decent_captures = 0;
  int player_piece_value;
  int null_reduction;
  n.top = &stack[0];
  floor = n.top;

  if(parent){
    n.parent = parent;
  }
  /* If enough nodes passed we should check the clock/input */
  if((negamax_nodes & 131071) == 0){
    get_input(0);
    check_time();
  }  

  /* Only check if not at root */
  if(check_draw(ply)){
    pline->cmove = 0;
    return DRAW;
  }


  if(in_check(!game.to_move)){
    pline->cmove = 0;
    return MATE-ply;
  }
  is_in_check =in_check(game.to_move);
  if(is_in_check)
    depth+=limit_extensions(CHECK_EXTENSION,ply);
  if(parent && parent->top->to_promote)
    depth += limit_extensions(PROMOTE_EXTENSION,ply);
  /* Look at the move stack to see if there was a recapture */
  if(ply >= 2 && data_stack_top[-2].victim &&  piece_value[data_stack_top[-2].victim+6] == piece_value[data_stack_top[-1].victim+6])
    depth += limit_extensions(RECAPTURE_EXTENSION,ply);


  if(depth <= 0){
    return quiescent(alpha,beta,pline,ply);
  }

  total_nodes++;
  negamax_nodes++;
  
  /* TT code*/
  if(tt[tt_key].zob == game.zob_key){
    if(tt[tt_key].depth >= depth/PLY){
      if(tt[tt_key].flag & TT_LOWER){
	if(tt[tt_key].s >= beta){
	  return tt[tt_key].s;
	}else if(tt[tt_key].s > alpha){
	  alpha = tt[tt_key].s;
	  
	  /*PV retrieval code*/
	  pline->argmove[0] = tt[tt_key].m;
	  pline->cmove = 1;
	  /* whenever there is a cutoff a null move won't do better */
  	  do_null_move = 0;
	}
      }else if(tt[tt_key].flag & TT_UPPER){
	/* this is buggy and not used */ 
	if(tt[tt_key].s <= alpha){
	  lm = alpha;
	  pline->cmove = 0;
	  return lm;
	} 
      }else{
	/*PV retreival code*/
	pline->argmove[0] = tt[tt_key].m;
	pline->cmove = 1;
	return tt[tt_key].s;
      }         
    }else{      
	/* if the depth is good enough for null move
	   and it didn't fail high null move won't
	   catch anything either
	   
	   This ain't really helping dude
	*/
      
      if((tt[tt_key].flag & TT_LOWER) == 0 &&
	 tt[tt_key].depth >= depth/PLY-NULL_PRUNE_R-1 &&
	 tt[tt_key].s < beta){
	do_null_move = 0;
      }
      
      
#ifdef _DEBUG
      if(!validate(tt[tt_key].m)){
	error("zob key crash");
      }
#endif
      if(!(tt[tt_key].flag & TT_UPPER)){
	*n.top = tt[tt_key].m;	      
	n.top++;
      }
    }
  }
  
  /* end of TT code*/
  
  

  /* Null move pruning code */
  /*  do_null_move = 0;*/

  if(do_null_move && 
     !is_in_check){
    if(game.to_move == WHITE)
      player_piece_value = W_Pieces_Material;
    else
      player_piece_value = B_Pieces_Material;
    
    if(depth > 2*PLY && player_piece_value >= 300){
      
      /* we need this to prevent null move to eat their own
	 pawn on an en_passant move */
      en_passant = game.en_passant;
      game.zob_key ^= en_passant_zob[(int)game.en_passant];
      game.en_passant = 0;
      game.zob_key ^= en_passant_zob[(int)game.en_passant];
      game.to_move = !game.to_move;
      game.zob_key ^= change_side_zob;
      do_null_move = 0;
      
      null_reduction = (depth >= 3 * PLY && player_piece_value >= 500) ? 3*PLY : 2*PLY;
      
      if(depth-null_reduction+limit_extensions(extensions,ply) > 0){
	lm = -negamax(depth-null_reduction+limit_extensions(extensions,ply),-beta,-beta+1,do_null_move,&dummy_line, ply+1,parent);
      }else{
	lm = -quiescent(-beta,-beta+1,&dummy_line,ply+1);
      }
      game.zob_key ^= en_passant_zob[(int)game.en_passant];
      game.en_passant = en_passant;
      game.zob_key ^= en_passant_zob[(int)game.en_passant];
      game.to_move = !game.to_move;
      game.zob_key ^= change_side_zob;
      if (lm >= beta) {
	/*addind the move with depth 0 gives the desired result of only
	  relying on the score */
	return lm;
      }else if(lm <= -(MATE-MAX_DEPTH))
	extensions += NMS_MATE_EXTENSION;
    }
  }

  do_null_move = 1;
  /* end of null move pruning code*/

  /* Test TT proposed move */
  if(n.top>floor){
    n.top--;
    make_move(*(n.top));
    lm = -negamax(depth-PLY+limit_extensions(extensions,ply),-beta,-alpha,do_null_move,&line,ply+1,&n);
    
    /* redundant search check code */
    already_search = *(n.top);
    
    undo_move(*(n.top));
    if(lm > alpha){
      if(lm >= beta){
	add_move_to_hh(*(n.top),depth/PLY);
	add_move_to_tt(*(n.top),lm,depth/PLY,TT_LOWER);
	return lm;
      }
      alpha = lm;
      /*PV retreival code*/
      pline->argmove[0] = *(n.top);
      memcpy(pline->argmove + 1, line.argmove,
	     line.cmove * sizeof(Move));
      pline->cmove = line.cmove + 1;
    }
  }else if(depth > 2*PLY && beta-alpha != 1){
    /* Internal Iterative deepening */
    /* Only do this on a PV node */
    dummy_line.cmove = 1;
    lm = negamax(depth-2*PLY,alpha,beta,0,&dummy_line,ply,parent);
    if(lm <= alpha){
      lm = negamax(depth-2*PLY,-MATE,beta,0,&dummy_line,ply+1,parent);
    }

    /* we have a result of MATE -MATE or DRAW. No need to search further */
    if(dummy_line.cmove == 0){
      pline->cmove = 0;
      /* we should return lm and not -lm because there was no move before the negamax call */
      return lm;
    }
    /* move should be in TT so lets retrieve it*/
    if(tt[tt_key].zob != game.zob_key){
      error("Error in internal iterative deepening, hash move was not there!");
    }
    *(n.top) = tt[tt_key].m;

    make_move(*(n.top));
    lm = -negamax(depth-PLY+limit_extensions(extensions,ply),-beta,-alpha,do_null_move,&line,ply+1,&n);

    /* redundancy check code */
    already_search = *(n.top);

    undo_move(*(n.top));
    if(lm > alpha){
      if(lm >= beta){
	add_move_to_hh(*(n.top),depth/PLY);
	add_move_to_tt(*(n.top),lm,depth/PLY,TT_LOWER);
	return lm;
      }
      alpha = lm;
      pline->argmove[0] = *(n.top);
      memcpy(pline->argmove + 1, line.argmove,
	     line.cmove * sizeof(Move));
      pline->cmove = line.cmove + 1;
    }
  }
  
  if(is_in_check){
    /* we have few to check so we can check all*/
    n.top = check_evasions(n.top,game.to_move);

    if(n.top == floor){
      /*if no moves return mate*/
      pline->cmove = 0;
      return -(MATE-ply);
    }else if(n.top == &(floor[1])){
      depth += limit_extensions(SINGLE_REPLY_EXTENSION,ply);
    }

    /* This can really cause problems */
    see_sort(n.top,floor);

    while(n.top>floor){
      n.top--;
      if(*((int*)n.top) == *((int *)&(already_search))){
	continue;
      }      
      make_move(*n.top);            
      lm = -negamax(depth-PLY+limit_extensions(extensions,ply),-alpha-1,-alpha,do_null_move,&line,ply+1,&n);
      if(lm > alpha && lm < beta && beta != alpha+1){
	lm = -negamax(depth-PLY+limit_extensions(extensions,ply),-beta,-alpha,do_null_move,&line,ply+1,&n);
      }
      undo_move(*n.top);
      if(lm > alpha){
	if(lm >= beta){
	  /* we have a cutoff */
	  add_move_to_hh(*n.top,depth/PLY);
	  add_move_to_tt(*n.top,lm,depth/PLY,TT_LOWER);

	  gkillers[ply][1] = *n.top;
	  gkillers_count[ply][1] = 0;

	  return lm;
	}
	alpha = lm;
	/*PV retreival code*/
	pline->argmove[0] = *n.top;
	memcpy(pline->argmove + 1, line.argmove,
	       line.cmove * sizeof(Move));
	pline->cmove = line.cmove + 1;
      }
    }
    if(alpha != init_alpha){
      add_move_to_hh(pline->argmove[0],depth/PLY);
      add_move_to_tt(pline->argmove[0],alpha,depth/PLY,0);
    }
    return alpha;            
  }


  
  if(game.to_move == WHITE){
    color = WHITE;
    n.top = all_w_cap(n.top);
  }else{
    color = BLACK;
    n.top = all_b_cap(n.top);
  }
  
  /*see sort*/
  decent_captures = see_sort(n.top,floor);

  /* Test decent captures */
  

  while(decent_captures){
    n.top--;
    decent_captures--;
    /* search redundancy avoidance code */
    /* I have to make this stupid cast so i can compare both structures binarily. It assumes int is 4 bytes */
    if(*((int *)n.top) == *((int *)&(already_search))){
      continue;
    }


    make_move(*n.top);          

    lm = -negamax(depth-PLY+limit_extensions(extensions,ply),-alpha-1,-alpha,do_null_move,&line,ply+1,&n);
    if(lm > alpha && lm < beta && beta != alpha+1){
      lm = -negamax(depth-PLY+limit_extensions(extensions,ply),-beta,-alpha,do_null_move,&line,ply+1,&n);
    }
    
    undo_move(*n.top);
    if(lm > alpha){
      if(lm >= beta){
	/* we have a cutoff */
	add_move_to_hh(*n.top,depth/PLY);
	add_move_to_tt(*n.top,lm,depth/PLY,TT_LOWER);
	return lm;
      }
      /* you should never trust that lm.m == *n.top
	 as this is not required from many functions*/
      alpha = lm;
      /*PV retreival code*/
      pline->argmove[0] = *n.top;
      memcpy(pline->argmove + 1, line.argmove,
	     line.cmove * sizeof(Move));
      pline->cmove = line.cmove + 1;
    }
  }

  /* Killer moves */

  for(i = 0;i<2;i++){ 
    if(validate(gkillers[ply][i])){
  /* Now lets do the futility pruning */
#ifdef _FUTILITY
      if(futility_prune(depth/PLY,is_in_check,gkillers[ply][i],alpha,ply)){
	continue;
      }
#endif

      if(*((int*)&gkillers[ply][i]) != *((int *)&(already_search))){
	make_move(gkillers[ply][i]);

	lm = -negamax(depth-PLY+limit_extensions(extensions,ply),-alpha-1,-alpha,do_null_move,&dummy_line,ply+1,&n);
	if(lm > alpha && lm < beta && beta != alpha+1){
	  lm = -negamax(depth-PLY+limit_extensions(extensions,ply),-beta,-alpha,do_null_move,&line,ply+1,&n);
	}
	
	undo_move(gkillers[ply][i]);
	if(lm > alpha){
	  if(lm >= beta){
	    /* we have a cutoff */
	    add_move_to_hh(gkillers[ply][i],depth/PLY);
	    add_move_to_tt(gkillers[ply][i],lm,depth/PLY,TT_LOWER);
	    gkillers_count[ply][i]++;
	    if(gkillers_count[ply][1] > gkillers_count[ply][0]){
	       m = gkillers[ply][0];
	       gkillers[ply][0] = gkillers[ply][1];
	       gkillers[ply][1] = m;
	    }
	    return lm;
	  }
	  alpha = lm;
	  /*PV retreival code*/
	  pline->argmove[0] = gkillers[ply][i];
	  memcpy(pline->argmove + 1, line.argmove,
		 line.cmove * sizeof(Move));
	  pline->cmove = line.cmove + 1;
	}
      }
    }       
  }
    


  /* We only have bad captures below n.top. Lets add all other moves */

  if(game.to_move == WHITE){
    n.top = all_w_non_cap(n.top);
  }else{
    n.top = all_b_non_cap(n.top);
  }
  
  /* history sort */
  history_sort_moves(floor,n.top-floor);

  
  while(n.top>floor){
    n.top--;
    /* search redundancy avoidance code */
    /* I have to make this stupid cast so i can compare both structures binarily. It assumes int is 4 bytes */
    if(*((int*)n.top) == *((int *)&(already_search))

      || *((int*)n.top) == *((int *)&(gkillers[ply][0])) ||
       *((int*)n.top) == *((int *)&(gkillers[ply][1]))

       ){
      continue;
    }

  /* Now lets do the futility pruning */
#ifdef _FUTILITY
    if(futility_prune(depth/PLY,is_in_check,*n.top,alpha,ply)){
      continue;
    }
#endif

    make_move(*n.top);
    lm = -negamax(depth-PLY+limit_extensions(extensions,ply),-alpha-1,-alpha,do_null_move,&dummy_line,ply+1,&n);
    if(lm > alpha && lm < beta && beta != alpha+1){
      lm = -negamax(depth-PLY+limit_extensions(extensions,ply),-beta,-alpha,do_null_move,&line,ply+1,&n);
    }
    undo_move(*n.top);
    if(lm > alpha){
      if(lm >= beta){
	/* we have a cutoff */
	add_move_to_hh(*n.top,depth/PLY);
	add_move_to_tt(*n.top,lm,depth/PLY,TT_LOWER);

	gkillers[ply][1] = *n.top;
	gkillers_count[ply][1] = 0;

	return lm;
      }

      alpha = lm;
      /*PV retreival code*/
      pline->argmove[0] = *n.top;
      memcpy(pline->argmove + 1, line.argmove,
	     line.cmove * sizeof(Move));
      pline->cmove = line.cmove + 1;
    }
  }

  /* check for stalemate */

  /* test position for stalemate
   
  setboard 8/8/8/1R3p2/5P2/2K3P1/7P/k7 w - - 0 56

  */ 

  /* maybe we should try to make some chekc for 
   alphas < than -(MATE-(ply+1)). They are a bug*/ 
  if(alpha == -(MATE-(ply+1)) && !is_in_check){
    /* we have a stalemate */
    pline->cmove = 0;
    return DRAW;    
  }
  if(alpha != init_alpha){
    add_move_to_hh(pline->argmove[0],depth/PLY);
    add_move_to_tt(pline->argmove[0],alpha,depth/PLY,0);
  }else{
    /*  This is causing so many bugs it's not even funny! */
    /*    add_move_to_tt(DUMMY_MOVE,alpha,depth,TT_UPPER);*/
  }
  return alpha;            
}

void clear_killers(){
  memset(gkillers,0,sizeof(Move)*MAX_DEPTH*2);
  memset(gkillers_count,0,sizeof(int)*MAX_DEPTH*2);
}


#ifdef _FUTILITY
int futility_prune(int depth,int is_in_check,Move m,int alpha, int ply){
  int material;
  /* lets make a more generous pruning*/
  if(!is_in_check && depth<3){
    if(game.to_move == WHITE){
      material = game.white_material-game.black_material;
    }else{
      material = game.black_material-game.white_material;
    }
    if(depth<2*PLY && material + FUT_MARGIN <= alpha){    
      return 1;
    }else if(depth<=3*PLY && material + RAZOR_MARGIN <= alpha){
      return 1;
    }
  }
  return 0;
}
#endif
