/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include "bitboard.h"
#include "gen_moves.h"
#include "search.h"
#include "score.h"
#include "make_moves.h"
#include "check.h"
#include "see.h"
#include "quiescent.h"
#include <chess_const.h>
#include <init.h>

/* VAI HAVER MERDA AQUI! */


int quiescent(int alpha, int beta,Line * pline, int ply){
  int good_moves;
  /*  int is_in_check;*/
  /*  int init_alpha = alpha;*/
  Move stack[MAX_POSSIBLE_MOVES];
  Move * floor = &(stack[0]);
  Move * top = floor;   
  int lm;  
  Line line;

  /* prepare for extensions */
  if(ply > MAX_PLY){
    return beta;
  }
  lm = score(alpha,beta);
  total_nodes++;
  quies_nodes++;
  if (lm >=  beta){
    return lm; 
  }else if( game.white_king == -1 || game.black_king == -1){
    pline->cmove = 0;
    /* we use ply-1 instead of ply because the mate was on the parent node */
    return -MATE+ply-1;
  }
    
  if(lm > alpha){
    alpha = lm;  
    /* No PV to retreive */
    pline->cmove = 0;
  }

  /*  is_in_check = in_check(game.to_move);*/

  if(game.to_move == WHITE){    
    if(in_check(WHITE)){
      top=check_evasions(top,WHITE);
      good_moves = top-floor;
    }else{
      top = all_w_cap(top);
      good_moves = see_sort(top,&stack[0]);
    }
  }else{
    if(in_check(BLACK)){
      top=check_evasions(top,BLACK);
      good_moves = top-floor;
    }else{
      top = all_b_cap(top);
      good_moves = see_sort(top,&stack[0]);
    }

    
    /*      top = all_b_cap(top);*/
  }
  

  /* Do NOT use minimal window
    
  if(!broad_search && good_moves){
    top--;
    good_moves--;
    make_move(*top);            
    lm = quiescent(-beta,-alpha);
    broad_search = 1;
    lm.s = -lm.s;
    undo_move(*top);
    if(lm.s > alpha){
      alpha = lm.s;
      res.s = lm.s;
      if (lm.s >=  beta){
	return lm;  
      }
    }
  }
  while( good_moves){
    top--;
    good_moves--;
    make_move(*top);
    lm = quiescent(-alpha-1,-alpha);
    lm.s = -lm.s;
    if(lm.s > alpha && lm.s < beta){
    lm = quiescent(-beta,-alpha);
      lm.s = -lm.s;
    }
    undo_move(*top);
    if(lm.s > alpha){
      alpha = lm.s;
      res.s = lm.s;
      if (lm.s >=  beta){
	return lm;  
      }
    }
  }
  */
  while( good_moves){
    top--;
    good_moves--;
    make_move(*top);
    lm = -quiescent(-beta,-alpha,&line,ply+1);
    undo_move(*top);
    if(lm > alpha){
      if (lm >=  beta){
	return lm;  
      }
      alpha = lm;
#ifndef NO_QUIES_PV
      /*PV retreival code*/
      pline->argmove[0] = *top;
      memcpy(pline->argmove + 1, line.argmove,
	     line.cmove * sizeof(Move));
      pline->cmove = line.cmove + 1;
#endif
    }
  }
  return alpha;
}


int mvv_lva_sort(Move * top,Move * floor){
  Move temp;
  Move * p;
  int ret = 0;
  while(floor<top){
  top--;
    for(p = floor;p < top;p++){
      if(piece_value[game.board[p->to]+6]-piece_value[game.board[p->from]+6] >
	 piece_value[game.board[(p+1)->to]+6]-piece_value[game.board[(p+1)->from]+6]){
	temp = *(p+1);
	*(p+1) = *p;
	*p = temp;
      }
    }
    if(piece_value[game.board[top->to]+6]-piece_value[game.board[top->from]+6] < 0){
      break;
    }
    ret++;
  }
  return ret;
}


int see_sort(Move * top, Move * floor){
  static int score[MAX_POSSIBLE_MOVES];  
  int tmp_score;
  int i;
  int ret = 0;
  Move tmp_move;
  Move * static_top = top;
  Move * p;
  top = floor;
  i = 0;
  while(top < static_top){
    if(top->special & EN_PASSANT)
      score[i] = 0;
    else if(top->to_promote)
      score[i] = piece_value[top->to_promote+6];       
    else if(piece_value[game.board[top->to]+6] - piece_value[game.board[top->from]+6] > 0)
      score[i] = piece_value[game.board[top->to]+6]-piece_value[game.board[top->from]+6]+1;
    else
      score[i] = SEE(top->from,top->to);
    top++;
    i++;
  }
  /* bubble sort */
  while(top > floor){
    top--;
    for(p = floor, i = 0;p < top; p++,i++){
      if(score[i] > score[i+1]){
	tmp_score = score[i];
	score[i] = score[i+1];
	score[i+1] = tmp_score;
	tmp_move = *p;
	*p = *(p+1);
	*(p+1) = tmp_move;
      }
    }
    /* Maybe this should be increased */
    if(score[top-floor] < 0){ 
      /* we're starting to sort bad captures, lets end here */
            break;
    }
    ret++;
  }
  /* return the number of non bad captures
   This is buggy if there are no bad captures!*/
  return ret;
}
    
    
