/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include <setjmp.h>

#include "bitboard.h"
#include "gen_moves.h"
#include "check.h"
#include "make_moves.h"
#include "search.h"
#include "util.h"
#include "controls.h"
#include "score.h"
#include "h_heuristic.h"
#include "tt.h"
#include "quiescent.h"
#include "validate.h"
#include "commands.h"
#include "interface.h"
#include <chess_const.h>
#include <init.h>
#include "zobrist.h"
#include "negamax.h"



static An_Move best_move={0,{0,0,0,0}};
int cur_depth = 0;
long total_nodes = 0;
long quies_nodes = 0;
long negamax_nodes = 0;
volatile char stop_search = 0;
static jmp_buf env;
static int doing_cleanup = 0;
int mate_in = 0;


static double time_i;
static double search_time;



An_Move search(){
  Position my_pos = game;
  int init_move_50_counter = move_50_counter;
  BitBoard * my_history_floor = history_floor;
  An_Move move;
  Line pv;
  Line line;
  double time_f;
  double time_middle;
  double prev_dtime = 0;
  double dtime;
  total_nodes = 0;
  quies_nodes = 0;
  negamax_nodes = 0;
  cur_depth = 1;
  stop_search = 0;

  shift_history();

  clear_killers();
  /* This will be used in the evaluation */
  start_piece_count = game.black_majors+game.black_minors;
  start_piece_count += game.white_majors+game.white_minors;
 
  if(mate_in){
    /*
      This should correct the bug related
      to getting the mate from the TT and then
      not getting enough depth in the next iteration
      The correct solution would be to use MAX_SCORE-
      MATE_DEPTH for MATE score :-/ but i'm lazy...
      Hopefully this should work...
    mate_in -= 2;
    cur_depth = mate_in;
    */
    mate_in = 0;
  }
  /* Init time */

  time_i = get_time();
  time_middle = time_i;
  time_f = time_i;
  
  /* if we're past move 40 move fast */
  if(t_control.fixed_time){
    search_time = t_control.fixed_time/100.0;
  }else{
    if(history_floor + move_50_counter - &move_history[0] > 80){
      /* Leave room for more 90 moves */
      search_time = t_control.timeleft/(90*100.0);
    }else{
      /* Leave room for more 30 moves */
      search_time = t_control.timeleft/(30*100.0);
    }
  }

  /* we'll exit the search by jumping here */
  setjmp(env);


  while(cur_depth < 90 && !stop_search){
    g_control.phase = SEARCHING;

    /*
      Search Strategy:
      Use a PVS search and rely on TT to be on the correct line on the first try
    */
    move.s = 
negamax(cur_depth*PLY+PLY/2,-1000000,1000000,1,&line,0,NULL);
    g_control.phase = 0;
    move.m = line.argmove[0];
    doing_cleanup = 1;

#ifdef _DEBUG
    if(check_board_corruption() == 1){
      error("Boards got corrupted!");
    }
#endif
    
    /* This will wipe out the move data stack */
    reset_data_stack();
    
    best_move = move;
    pv = line;
    stuff_pvs_to_tt(&pv,best_move.s,cur_depth);
    time_f = get_time();
    dtime = time_f-time_middle;
    if(!x_control.silent && !x_control.use_xboard){
      printf("Time: %fs\n",time_f-time_middle);      
      print_pvs(&pv,stdout);
      printf("Score: %d\n",best_move.s);
    }else if(x_control.use_xboard && x_control.thinking){
      printf("%d %d %d %ld ",cur_depth,best_move.s,(int)((time_f-time_i)*100),total_nodes);
      print_pvs(&pv,stdout);
    }

    if(prev_dtime > 0.3){
      fprintf(logfile,"DTime - %f D - %d EBF - %f\n",dtime,cur_depth,dtime/prev_dtime);
      fflush(logfile);
    }
    time_middle = time_f;
    prev_dtime = dtime;

    /*    if(tt[game.zob_key&n_bit_mask[TRANS_SZ]].depth < cur_depth || tt[game.zob_key&n_bit_mask[TRANS_SZ]].zob != game.zob_key){*/
    /*    stuff_pvs_to_tt(&pv,best_move.s,cur_depth);*/
      /*    }*/
    reset_data_stack();
    /* we found the mate */
    if(best_move.s >= MATE-MAX_DEPTH && !mate_in){
      /* the -3 is because the MATE score reported means that the king was captured so you have to remove 2,
	 the other 1 is because when the score is returned ElTurco still didn't make any move, yet the mate
	 in is reported after the move */
      mate_in = MATE-best_move.s-3;
    }else if(best_move.s <= -(MATE-MAX_DEPTH)){
      /* we just got mated */
      cur_depth++;
      doing_cleanup = 0;
      break;
    }
    /* if we find a mate or we only have less than half the search time available don't try to 
       search for another ply*/
    if((best_move.s >= MATE-MAX_DEPTH) || (time_f-time_i > (float)t_control.timeleft/(30.0*2.0*100.0) &&
				t_control.fixed_time == 0)){
      /* the cur_depth++ is only here because the other functions assume that this functions
	 returns in the middle of a search and only the previous depth is valid which is not the case */
      cur_depth++;
      doing_cleanup = 0;
      break;
    }
    cur_depth++;
    doing_cleanup = 0;
  }
  game = my_pos;
  move_50_counter = init_move_50_counter;
  history_floor = my_history_floor;

  /* This will wipe out the move data stack */
  reset_data_stack();

  if(cur_depth == 0 && !mate_in){
    error("Couldn't find a move!");
    /*    exit(1);*/
  }
  time_f = get_time();
  if(!x_control.silent){
    printf("Depth: %d  Time: %fs\n",cur_depth-1,time_f-time_i);
    printf("Nodes per second: %d\n",(int)(total_nodes/(time_f-time_i)));
    printf("Quiescent Nodes per second: %d\n",(int)(quies_nodes/(time_f-time_i)));
    print_pvs(&pv,stdout);
  }
  fprintf(logfile,"Depth: %d  Time: %fs\n",cur_depth-1,time_f-time_i);
  fprintf(logfile,"Nodes per second: %d\n",(int)(total_nodes/(time_f-time_i)));
  fprintf(logfile,"Quiescent Nodes per second: %d\n",(int)(quies_nodes/(time_f-time_i)));
  print_pvs(&pv,logfile);

  stuff_pvs_to_tt(&pv,best_move.s,cur_depth-1);
  /* This will wipe out the move data stack */
  /*  reset_data_stack();*/


  return best_move;
}


void end_search(){
  stop_search = 1;
  longjmp(env,1);
}


void check_time(){
  if(get_time() > time_i+search_time)
    end_search();
}
