/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include "bitboard.h"
#include <gen_moves.h>
#include "check.h"
#include "attacks.h"
#include "score.h"
#include "validate.h"
#include "see.h"
#include <chess_const.h>
#include <math_util.h>
#include <init.h>


/* returns the value of capturing the piece in target square
   with the piece on source square. */

/* It uses the exact same algorythm as Crafty */
int SEE(int source, int target) {
  BitBoard attackers;
  BitBoard xray;
  int score_list[32];
  int color;
  int i = 1;
  int last_attacker_value;
  int square = -1;

  if(source < 0){
    color = BLACK;
  }else{
    color = WHITE;
  }
  attackers = gen_attackers(target);

  last_attacker_value = piece_value[game.board[source]+6];

  score_list[0] = piece_value[game.board[target]+6];
  attackers ^= mask[source];
  /* add to the list a possible piece hiding behind the attacker */
  xray = get_xray(target,source);
  if(xray)
    attackers |= xray;

  color ^= 1; /* swap color */

  /* pick the least valuable attacker */
  while (attackers) {
    if (color == WHITE) {
      if (WPawns & attackers)
        square = get_first_lsb(WPawns & attackers);
      else if (WKnights & attackers)
        square=get_first_lsb(WKnights & attackers);
      else if (WBishops & attackers)
        square=get_first_lsb(WBishops & attackers);
      else if (WRooks & attackers)
        square=get_first_lsb(WRooks & attackers);
      else if (WQueens & attackers)
        square=get_first_lsb(WQueens & attackers);
      else if (mask[game.white_king] & attackers)
        square=game.white_king;
      else
	/* we could have only black attackers */
	break;
    }else{
      if (BPawns & attackers)
        square=get_first_lsb(BPawns & attackers);
      else if (BKnights & attackers)
        square=get_first_lsb(BKnights & attackers);
      else if (BBishops & attackers)
        square=get_first_lsb(BBishops & attackers);
      else if (BRooks & attackers)
        square=get_first_lsb(BRooks & attackers);
      else if (BQueens & attackers)
        square=get_first_lsb(BQueens & attackers);
      else if (mask[game.black_king] & attackers)
        square=game.black_king;
      else
	/* we could have only white attackers */
	break;
    }

    /* remove it from the list and find new attackers */
    score_list[i] = -score_list[i-1]+last_attacker_value;  

    last_attacker_value = piece_value[game.board[square]+6];
    attackers ^= mask[square];

    xray = get_xray(target,square);
    if(xray)
      attackers |= xray;

    color ^= 1; /* swap color */
    i++;
  }
    
  /* we need a minimax procedure to find where the captures should stop */
  while(--i)
    if(score_list[i] > -score_list[i-1]) score_list[i-1]=-score_list[i];
  return (score_list[0]);
}

/* find any attacker in the beam a-b******* */
BitBoard get_xray(int a, int b){
  int direc;
  BitBoard board = 0;
  direc = directions[a][b];
  if(direc == FILE_UP || direc == FILE_DOWN){
    board = attacks_file(b) & beam[a][b] & (BQueens | WQueens | BRooks |WRooks);
  }else if(direc == RANK_LEFT || direc == RANK_RIGHT){
    board = attacks_rank(b) & beam[a][b] & (BQueens | WQueens | BRooks |WRooks);
  }else if(direc == ODD_UP || direc == ODD_DOWN){
    board = attacks_odd_diag(b) & beam[a][b] & (BBishops | BQueens | WBishops | WQueens);
  }else if(direc == EVEN_UP || direc == EVEN_DOWN){
    board = attacks_even_diag(b) & beam[a][b] & (BBishops | BQueens | WBishops | WQueens);
  }
  return board;
}
