/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include "bitboard.h"
#include <gen_moves.h>
#include <search.h>
#include "util.h"
#include "make_moves.h"
#include "tt.h"
#include <init.h>
#include <controls.h>

Move DUMMY_MOVE = {0,61,0,0};

TT_Entry * tt= NULL;
Pawn_Entry * pawn_table= NULL;
int _TRANS_SZ = 262144;
int TRANS_SZ;

void init_tt(){
  BitBoard i = 1;
  BitBoard c;
  BitBoard p;
  int tmp = 0;
  TRANS_SZ = _TRANS_SZ;
  while(TRANS_SZ>>1){
    tmp++;
    TRANS_SZ = TRANS_SZ>>1;
  }  
  TRANS_SZ = tmp;
  c = i<<TRANS_SZ;
  p = i<<PAWN_SZ;

  if(!x_control.silent){
    printf("Transposition table has %llu entries (%llu bytes)\n",c,c*18);
  }
  
  if(!tt)
    tt = (TT_Entry *)malloc(sizeof(TT_Entry)*c);
  if(!pawn_table)
    pawn_table = (Pawn_Entry *)malloc(sizeof(Pawn_Entry)*c);
  /* this makes sure t*/
  tt[0].zob = 1;
  pawn_table[0].zob = 1;
  tt[0].depth = -1;
  for(i =1;i<c;i++){
    tt[i].zob = 0;
    pawn_table[i].zob = 0;
    tt[i].depth = -1;
  }
}

void add_move_to_tt(Move m,int score, char depth, char flag){ 
  BitBoard i;
  i = game.zob_key & n_bit_mask[TRANS_SZ];
#ifdef _DEBUG
  if(m.to != 0 || m.from != 0){
#endif
    tt[i].zob = game.zob_key;
    tt[i].m = m;
    tt[i].s = score;     
    tt[i].depth = depth;
    tt[i].flag = flag;
#ifdef _DEBUG
  }else{
    warn("trying to hash a bad move");
  }
#endif
}

void stuff_pvs_to_tt(Line * pv,int score,int depth){
  int i;
  for(i = 0;i<pv->cmove && i<=depth;i++){
    add_move_to_tt(pv->argmove[i],score,depth-i,0);
    make_move(pv->argmove[i]);
  }    
  for(;i>0;i--){
    undo_move(pv->argmove[i-1]);
  }
}

void add_move_to_pt(int score){ 
  BitBoard i;
  i = game.pawn_key & n_bit_mask[PAWN_SZ];
  pawn_table[i].zob = game.pawn_key;
  pawn_table[i].s = score;     
}
