/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#ifndef _TT_H_
#define _TT_H_

/* this defines the size of the Transition table in terms of bits */
/* the real size is 18*TRANS_SZ */

extern int _TRANS_SZ;



/* Pawn table size. This table is simpler an only uses 1 int and 1 BitBoard for each position so the size
   is simple 12*2^PAWN_SZ */
#ifndef PAWN_SZ
#define PAWN_SZ 18 /* 1024KB */
#endif





typedef struct{
  Move m;
  short s;
  char depth;
  char flag;
  BitBoard zob;
}TT_Entry;



typedef struct {
  BitBoard  zob;
  int s;
  /*  unsigned char allb;
  unsigned char black_defects_k;
  unsigned char black_defects_q;
  unsigned char passed_b;
  unsigned char candidates_b;
  unsigned char allw;
  unsigned char white_defects_k;
  unsigned char white_defects_q;
  unsigned char passed_w;
  unsigned char candidates_w;
  unsigned char protected;
  unsigned char outside;
  unsigned char open_files; */
} Pawn_Entry;

extern TT_Entry * tt;
extern Pawn_Entry * pawn_table;
extern Move DUMMY_MOVE;
extern int TRANS_SZ;

void init_tt();
int lookup_pawn_table();

void add_move_to_tt(Move m, int s,char depth, char flag);
void stuff_pvs_to_tt(Line * pv,int score,int depth);


/* what is stored is the lower bound of the real value 
   and came from a fail high */
#define TT_LOWER 2
/* what is stored is the upper bound of the real value 
   and came from a fail low */
#define TT_UPPER 4

#endif /* _TT_H_ */
