/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include <unistd.h>
#include <ctype.h>
#include <bitboard.h>
#include <gen_moves.h>
#include <interface.h>
#include <search.h>
#include <elturco.h>
#include <chess_const.h>
#include <score.h>
#include <util.h>
#include <init.h>
#include <zobrist.h>
#include <make_moves.h>
#include <controls.h> 
#include <tt.h>

#ifndef _WIN32
#include <sys/times.h>
#include <sys/time.h>
#endif
/*
  I'm gonna use ftime(2) because it's the only function available
  on windows unix and mac for better than second accurancy... arrest me
  if you want!
*/
#include <sys/timeb.h>

/*don't ask*/
int getopt(int argc, char * const argv[],
                  const char *optstring);

char silent = 0;
char init_position = 1;

void call_error(char * file, int line, char * string){
  FILE * fd;
  fd = fopen("error.log","a");
  fprintf(stderr,"%s:%d %s\n",file,line,string);
  fprintf(fd,"%s:%d %s\n",file,line,string);
  fclose(fd);
  fprintf(stdout,"resign\n");
}

void call_warn(char * file, int line, char * string){
  FILE * fd;
  fd = fopen("warn.log","a");
  fprintf(stderr,"%s:%d %s\n",file,line,string);
  fprintf(fd,"%s:%d %s\n",file,line,string);
  fclose(fd);
}




void print_moves(Move * top,Move * floor){
  int i;
  int n = 1;
  Move * orig_top = top;
  FILE * fd;
  fd = stdout;
  while(top > floor){
    fprintf(fd,"Move:%d\n",n);
    n++;
    top--;
    for(i = 0;i<64;i++){
      if(top->from == i){
	fprintf(fd,"F");
      }else if(top->to == i){
	fprintf(fd,"T");
      }else{
	fprintf(fd,"_");
      }
      fprintf(fd," ");
      if(!((i+1)%8)){
	fprintf(fd,"\n");
      }
    }
    fprintf(fd,"\n"); 
  }
  fd = fopen("error.log","a");
  top = orig_top;
  while(top > floor){
    fprintf(fd,"Move:%d\n",n);
    n++;
    top--;
    for(i = 0;i<64;i++){
      if(top->from == i){
	fprintf(fd,"F");
      }else if(top->to == i){
	fprintf(fd,"T");
      }else{
	fprintf(fd,"_");
      }
      fprintf(fd," ");
      if(!((i+1)%8)){
	fprintf(fd,"\n");
      }
    }
    fprintf(fd,"special - %d\n",top->special); 
    fprintf(fd,"promote - %d\n",top->to_promote); 
    fprintf(fd,"\n"); 
  }
  fclose(fd);  
}


void print_board(FILE * fd){
  Position * p = &game;
  int i;
  if(p->to_move){
    fprintf(fd,"To move: Black\n");
  }else{
    fprintf(fd,"To move: White\n");
  }
  if(p->comp == WHITE){
    fprintf(fd,"Computer playing White\n");
  }else{
    fprintf(fd,"Computer playing Black\n");
  }
  if(p->en_passant){
    fprintf(fd,"En passant target in %c%d\n",'a'+p->en_passant%8,8-p->en_passant/8);
  }
  fprintf(fd,"Material Imbalance: %d\n",p->white_material-p->black_material);
  for(i = 0;i<64;i++){
    if(p->board[i] == w_pawn){
      fprintf(fd,"P");
    }else if(p->board[i] == b_pawn){
      fprintf(fd,"p");
    }else if(p->board[i] == w_rook){
      fprintf(fd,"R");
    }else if(p->board[i] == b_rook){
      fprintf(fd,"r");
    }else if(p->board[i] == w_knight){
      fprintf(fd,"N");
    }else if(p->board[i] == b_knight){
      fprintf(fd,"n");
    }else if(p->board[i] == w_bishop){
      fprintf(fd,"B");
    }else if(p->board[i] == b_bishop){
      fprintf(fd,"b");
    }else if(p->board[i] == w_queen){
      fprintf(fd,"Q");
    }else if(p->board[i] == b_queen){
      fprintf(fd,"q");
    }else if(p->board[i] == w_king){
      fprintf(fd,"K");
    }else if(p->board[i] == b_king){
      fprintf(fd,"k");
    }else if(p->board[i] == none){
      fprintf(fd,"_");
    }
    fprintf(fd," ");
    if(!((i+1)%8)){
      fprintf(fd,"\n");
    }
  }
}


     
  
void load_board(char * string){
  int c;
  int i = 0;
  int j = 0;
  int k = 0;
  BitBoard a = 1;
  init_position = 0;
  mate_in = 0;
  game.to_move = BLACK;
  game.en_passant = 0;
  game.new_castle = 0;
  game.white_material= 0;
  game.black_material = 0;
  game.white_majors = 0;
  game.black_majors = 0;
  game.white_minors = 0;
  game.black_minors = 0;
  game.white_pawns = 0;
  game.black_pawns = 0;
  WPawns = 0;
  WKnights = 0;
  WBishops = 0;
  WRooks = 0;
  WQueens = 0;
  game.w_pieces = 0;
  BPawns = 0;
  BKnights = 0;
  BBishops = 0;
  BRooks = 0;
  BQueens = 0;
  game.b_pieces = 0;
  game.all_r_90 = 0;
  game.all_even_diag = 0;
  game.all_odd_diag = 0;
  for(j = 0;j<72 && i<64;j++){
    if(string[j] == ' ' || string[j] == '/')
      continue;
    c = string[j];
    if(c == 'P'){
      WPawns |= mask[i];
      game.white_material += PAWN_VALUE;
      game.white_pawns++;
      game.w_pieces |= mask[i];
      game.board[i] = w_pawn;
    }else if(c == 'p'){
      BPawns |= mask[i];
      game.black_material += PAWN_VALUE;
      game.black_pawns++;
      game.b_pieces |= mask[i];
      game.board[i] = b_pawn;
    }else if(c == 'R'){
      WRooks |= mask[i];
      game.white_majors++;
      game.white_material+= ROOK_VALUE;
      game.w_pieces |= mask[i];
      game.board[i] = w_rook;
    }else if(c == 'r'){
      BRooks |= mask[i];
      game.black_majors++;
      game.black_material += ROOK_VALUE;
      game.b_pieces |= mask[i];
      game.board[i] = b_rook;
    }else if(c == 'N'){
      WKnights |= mask[i];
      game.white_minors++;
      game.white_material += KNIGHT_VALUE;
      game.w_pieces |= mask[i];
      game.board[i] = w_knight;
    }else if(c == 'n'){
      BKnights |= mask[i];
      game.black_minors++;
      game.black_material += KNIGHT_VALUE;
      game.b_pieces |= mask[i];
      game.board[i] = b_knight;
    }else if(c == 'B'){
      WBishops |= mask[i];
      game.white_minors++;
      game.white_material += BISHOP_VALUE;
      game.w_pieces |= mask[i];
      game.board[i] = w_bishop;
    }else if(c == 'b'){
      BBishops |= mask[i];
      game.black_minors++;
      game.black_material += BISHOP_VALUE;
      game.b_pieces |= mask[i];
      game.board[i] = b_bishop;
    }else if(c == 'Q'){
      WQueens |= mask[i];
      game.white_majors++;
      game.white_material+= QUEEN_VALUE;
      game.w_pieces |= mask[i];
      game.board[i] = w_queen;
    }else if(c == 'q'){
      BQueens |= mask[i];
      game.black_majors++;
      game.black_material+= QUEEN_VALUE;
      game.b_pieces |= mask[i];
      game.board[i] = b_queen;
    }else if(c == 'K'){
      game.white_king = i;
      game.w_pieces |= mask[i];
      game.board[i] = w_king;
    }else if(c == 'k'){
      game.black_king = i;
      game.b_pieces |= mask[i];
      game.board[i] = b_king;
    }else if(c == '-'){
      game.board[i] = none;
    }else if(isdigit(c)){
      for(k = 0;k<c-(int)'0'-1;k++){
	game.board[i] = none;
	i++;
      }
      game.board[i] = none;
    }	    
    i++;
  }
  for(i = 0;i<64;i++){
    if(game.board[i]){
      game.all_odd_diag |= a << (odd_diag_map[i]) ;
      game.all_even_diag |= a << (even_diag_map[i]) ;
      game.all_r_90 |= a << (r_90_map[i]);

    }
  }
  j++;
  if(string[j] == 'B' || string[j] == 'b'){
    game.to_move = BLACK;
  }else if(string[j] == 'W'|| string[j] == 'w'){
    game.to_move = WHITE;
  }
  j += 2;
  if(isdigit(string[j])){
    /* probably type 12 FICS format */
    if(string[j] != '0'){
      game.new_castle |= W_SMALL_CASTLE;
    }
    j++;
    if(string[j] != '0'){
      game.new_castle |= W_BIG_CASTLE;
    }
    j ++;
    if(string[j] != '0'){
      game.new_castle |= B_SMALL_CASTLE;
    }
    j++;
    if(string[j] != '0'){
      game.new_castle |= B_BIG_CASTLE;
    }
    j += 2;
    if(string[j] == '-'){
      /* no en passant */
      j++;
      game.en_passant = 0;
    }else{
      if(game.to_move == WHITE){
	game.en_passant = 8+(int)string[j]-(int)'0';
      }else if(game.to_move == BLACK){
	game.en_passant = 48+(int)string[j]-(int)'0';
      }
    }
  }else{
    /* regular FEN format */
    if(string[j] == '-'){
      /* no castling available */
      j++;
    }
    if(string[j] == 'K'){
      game.new_castle |= W_SMALL_CASTLE;
      j++;
    }
    if(string[j] == 'Q'){
      game.new_castle |= W_BIG_CASTLE;
      j++;
    }
    if(string[j] == 'k'){
      game.new_castle |= B_SMALL_CASTLE;
      j++;
    }
    if(string[j] == 'q'){
      game.new_castle |= B_BIG_CASTLE;
      j++;
    }
    j++;
    if(string[j] == '-'){
      game.en_passant = 0;
    }else{
#ifdef _DEBUG
      printf("%c%c\n",string[j],string[j+1]);
      printf("enpassant- %d\n",string[j]-'a'+8*(8-(string[j+1]-(int)'0')));
#endif
      game.en_passant = string[j]-'a'+8*(8-(string[j+1]-(int)'0'));
    }
  }
    
  game.zob_key = gen_zob_key();  
  game.pawn_key = gen_pawn_key();
  g_control.moves_out_of_book = 0;
}




void parse_args(int argc, char ** argv){
  int o;
  setbuf(stdout,NULL);
/*  setbuf(stdin,NULL); */
  for(o = getopt(argc,argv,"bwsrt:");o != -1;o = getopt(argc,argv,"bwsrt:")){
    if(o == 'b'){
      game.comp = BLACK;
    }
    if(o == 'w'){
      game.comp = WHITE;
    }
    if(o == 's'){
      silent = 1;
    }
    if(o == 't'){
      _TRANS_SZ = atoi(optarg);
    }
    if(o == 'r'){
      g_control.quick_resign = 1;
    }

  }

}      




void chomp(char * str){
  while(str[strlen(str)-1] == '\r' 
	|| str[strlen(str)-1] == '\n' 
	|| str[strlen(str)-1] == ' ')
    str[strlen(str)-1] = '\0';	      
}

void print_pgn_header(){
  struct tm *  p_time;
  time_t sec;
  time(&sec);
  p_time = localtime(&sec);
  fprintf(g_control.pgn,"\n\n[Date \"%d.%02d.%d\"]\n",p_time->tm_year+1900,p_time->tm_mon,p_time->tm_mday);
  fprintf(g_control.pgn,"[Time \"%d:%d:%d\"]\n",p_time->tm_hour,p_time->tm_min,p_time->tm_sec);
  if(game.to_move == game.comp){
    fprintf(g_control.pgn,"[White \"ElTurco %s\"]\n",VERSION);
    if(g_control.opponent[0])
      fprintf(g_control.pgn,"[Black \"%s\"]\n",g_control.opponent);
  }else{
    if(g_control.opponent[0])
      fprintf(g_control.pgn,"[White \"%s\"]\n",g_control.opponent);
    fprintf(g_control.pgn,"[Black \"ElTurco %s\"]\n",VERSION);
  }
  if(g_control.elo[0])
    fprintf(g_control.pgn,"[WhiteElo \"%d\"]\n",g_control.elo[0]);
  if(g_control.elo[1])
    fprintf(g_control.pgn,"[BlackElo \"%d\"]\n",g_control.elo[1]);
  if(t_control.starting_time){
    if(t_control.increment){
      fprintf(g_control.pgn,"[TimeControl \"%d+%d\"]\n",t_control.starting_time/100,t_control.increment/100);
    }else{
      fprintf(g_control.pgn,"[TimeControl \"%d\"]\n",t_control.starting_time/100);
    }
  }
  fprintf(g_control.pgn,"\n");
}

void open_log(){
  logfile = fopen("el_turco.log","a");
  if(!logfile){
    fprintf(stderr,"Unable to open logfile!\n");
    exit(1);
   }    
}

void close_log(){
  fclose(logfile);
}


void print_bitboard(BitBoard p){
  BitBoard a;
  int i;
  a = 1;
  for (i = 0;i<64;i++){
    if(a&p)
      printf("X");
    else
      printf("-");
    printf(" ");    
    if(!((i+1)%8))
      printf("\n");
    p = p >> 1;
  }
}

void print_pvs(Line * pv,FILE * fp){
  int i;
  if(pv->cmove){
    if(x_control.use_xboard && x_control.thinking){
      fprintf(fp,"%s",move2algebraic(pv->argmove[0],&game));
    }else{
      fprintf(fp,"PVS-%s",move2algebraic(pv->argmove[0],&game));
    }
    make_move(pv->argmove[0]);
    for(i = 1;i<pv->cmove;i++){
      fprintf(fp," %s",move2algebraic(pv->argmove[i],&game));
      make_move(pv->argmove[i]);
    }
    for(i = pv->cmove;i>0;i--){
      undo_move(pv->argmove[i-1]);
    }
    fprintf(fp,"\n");
  }
}


double get_time(){
#ifdef _WIN32
  struct timeb tb;
  ftime(&tb);
  return tb.time+tb.millitm/1000.0;
#else
  struct timeval tp;
  struct timezone tzp;
  gettimeofday(&tp,&tzp);
  return tp.tv_sec+tp.tv_usec/1000000.0;
#endif
}

void cross_fread_uint32(unsigned int * result,  FILE *stream)
{
  unsigned char buffer[4];
  fread(buffer, 4, 1, stream);
  *result = (buffer[3] << 24) | (buffer[2] << 16) | (buffer[1] << 8) | (buffer[0]);
}

void cross_fread_book_positions(BookPosition * result,int n,FILE *stream)
{
  int i;
  unsigned char buffer[20];
  for(i = 0;i<n;i++){
    fread(buffer, 20, 1, stream);
    result->position = ((BitBoard)buffer[7] << 56) | ((BitBoard)buffer[6] << 48) | ((BitBoard)buffer[5] << 40) | ((BitBoard)buffer[4] << 32)|
      ((BitBoard)buffer[3] << 24) | ((BitBoard)buffer[2] << 16) | ((BitBoard)buffer[1] << 8) | ((BitBoard)buffer[0]);
    result->status_played =  (buffer[11] << 24) | (buffer[10] << 16) | (buffer[9] << 8) | (buffer[8]);
    result++;
  }
}
