/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include "bitboard.h"
#include "gen_moves.h"
#include <search.h>
#include "util.h"
#include "validate.h"
#include <chess_const.h>
#include <init.h>
#include <math_util.h>

unsigned char val_table[13][64][64];
/*unisgned char const *** vt_p = &val_table[6][0][0];*/

void init_val_table(){
  int p,from,to;
  
  for(p = -6;p<7;p++){
    for(from = 0;from<64;from++){
      for(to = 0;to<64;to++){
	if(abs(p) == 6){
	  /* KING */
	  if(abs(distance(from,to)) == 1){
	    if(ROW(from) == ROW(to)){
	      if(from < to){
		val_table[p+6][from][to] =RANK_RIGHT;
	      }else{
		val_table[p+6][from][to] =RANK_LEFT;
	      }
	    }else if(COL(from) == COL(to)){
	      if(from > to){
		val_table[p+6][from][to] = FILE_UP;
	      }else{
		val_table[p+6][from][to] = FILE_DOWN;
	      }
	    }else if(ROW(from)-ROW(to) == COL(from) - COL(to)){
	      if(from > to){
		val_table[p+6][from][to] = EVEN_UP;
	      }else{
		val_table[p+6][from][to] = EVEN_DOWN;
	      }
	    }else if(ROW(from)-ROW(to) == -(COL(from) - COL(to))){
	      if(from > to){
		val_table[p+6][from][to] = ODD_UP;
	      }else{
		val_table[p+6][from][to] = ODD_DOWN;
	      }
	    }
	  }else{
	    val_table[p+6][from][to] = INVALID_MOVE;
	  }
	}else if(abs(p) == 5){
	  /* QUEEN */
	  if(from/8 == to/8){
	    if(from < to){
	      val_table[p+6][from][to] =RANK_RIGHT;
	    }else{
	      val_table[p+6][from][to] =RANK_LEFT;
	    }
	  }else if(from %8 == to%8){
	    if(from > to){
	      val_table[p+6][from][to] = FILE_UP;
	    }else{
	      val_table[p+6][from][to] = FILE_DOWN;
	    }
	  }else if(from/8 -to/8 == (from%8) - (to%8)){
	    if(from > to){
	      val_table[p+6][from][to] = EVEN_UP;
	    }else{
	      val_table[p+6][from][to] = EVEN_DOWN;
	    }
	  }else if(from/8-to/8 == -((from%8) - (to%8))){
	    if(from > to){
	      val_table[p+6][from][to] = ODD_UP;
	    }else{
	      val_table[p+6][from][to] = ODD_DOWN;
	    }
	  }else{
	    val_table[p+6][from][to] = INVALID_MOVE;
	  }
	}else if(abs(p) == 4){
	  /*BISHOP*/
	  if(from/8 -to/8 == (from%8) - (to%8)){
	    if(from > to){
	      val_table[p+6][from][to] = EVEN_UP;
	    }else{
	      val_table[p+6][from][to] = EVEN_DOWN;
	    }
	  }else if(from/8-to/8 == -((from%8) - (to%8))){
	    if(from > to){
	      val_table[p+6][from][to] = ODD_UP;
	    }else{
	      val_table[p+6][from][to] = ODD_DOWN;
	    }
	  }else{
	    val_table[p+6][from][to] = INVALID_MOVE;
	  }          
	}else if(abs(p) == 3){
	  /* KNIGHT */
	  if((abs(from/8-to/8) == 2 && abs((from%8) - (to%8)) == 1) ||
	     (abs(from/8-to/8) == 1 && abs((from%8) - (to%8)) == 2) ){
	    val_table[p+6][from][to] = KNIGHT_MOVE;
	  }else{
	    val_table[p+6][from][to] = INVALID_MOVE;
	  }
	}else if(abs(p) == 2){
	  /* ROOK */
	  if(from/8 == to/8){
	    if(from < to){
	      val_table[p+6][from][to] =RANK_RIGHT;
	    }else{
	      val_table[p+6][from][to] =RANK_LEFT;	  
	    }
	  }else if(from %8 == to%8){
	    if(from > to){
	      val_table[p+6][from][to] = FILE_UP;
	    }else{
	      val_table[p+6][from][to] = FILE_DOWN;
	    }
	  }else{
	    val_table[p+6][from][to] = INVALID_MOVE;
	  }
	}else if(p == 1){
	  /* WHITE PAWN */
	  if(to == from -16 && from /8 == 6){
	    val_table[p+6][from][to] = PAWN_DOUBLE_UP;
	  }else if(to== from-8){
	    val_table[p+6][from][to] = PAWN_NORMAL_UP;
	  }else if(to/8 - from/8 == -1 && (to%8) -(from %8) == -1){
	    val_table[p+6][from][to] = PAWN_EVEN_CAP_UP;
	  }else if(to/8 - from/8 == -1 && (to%8) -(from %8) == 1){
	    val_table[p+6][from][to] = PAWN_ODD_CAP_UP;
	  }else{
	    val_table[p+6][from][to] = INVALID_MOVE;
	  }
	}else if(p == -1){
	  /* BLACK PAWN */
	  if(to == from +16 && from /8 == 1){
	    val_table[p+6][from][to] = PAWN_DOUBLE_DOWN;
	  }else if(to== from+8){
	    val_table[p+6][from][to] = PAWN_NORMAL_DOWN;
	  }else if(to/8 - from/8 == 1 && (to%8) -(from %8) == 1){
	    val_table[p+6][from][to] = PAWN_EVEN_CAP_DOWN;
	  }else if(to/8 - from/8 == 1 && (to%8) -(from %8) == -1){
	    val_table[p+6][from][to] = PAWN_ODD_CAP_DOWN;
	  }else{
	    val_table[p+6][from][to] = INVALID_MOVE;
	  }
	}else if (p == 0){
	  val_table[p+6][from][to] = INVALID_MOVE;
	}		
	if(from == CASTLE_FROM && to == CASTLE_TO){
	  val_table[p+6][from][to] = POSSIBLY_CASTLE;
	}
      }
    }
  }
}      
	  
	    
int validate(Move m){
  unsigned char key;
  int i;

  /* for the castles */
  Move stack[100];
  Move * my_top = &(stack[0]);

  key = val_table[game.board[m.from]+6][m.from][m.to];

  if(key == INVALID_MOVE){
    return 0;
  }



  if(((game.to_move == WHITE && game.board[m.from] <= 0) || (game.to_move == BLACK && game.board[m.from] >= 0)) && !(m.special & (W_SMALL_CASTLE|W_BIG_CASTLE|B_SMALL_CASTLE|B_BIG_CASTLE))){
    return 0;
  }
  if((m.special & CAPTURE && !game.board[m.to] && !(m.special & EN_PASSANT)) ||( !(m.special & CAPTURE) && game.board[m.to] && !(m.special & (W_SMALL_CASTLE|W_BIG_CASTLE|B_SMALL_CASTLE|B_BIG_CASTLE)))){
    return 0;
  }

  if((m.special & IRREVERSIBLE) && !(m.special & CAPTURE) && game.board[m.from] != w_pawn && game.board[m.from] != b_pawn){
    return 0;
  }else if(!(m.special & IRREVERSIBLE) && (game.board[m.from] == w_pawn || game.board[m.from] == b_pawn)){
    return 0;
  }
  
  if(m.special & EN_PASSANT && m.to != game.en_passant){
    return 0;
  }

  if(m.to_promote && game.board[m.from] != w_pawn && game.board[m.from] != b_pawn){
    return 0;
  }


  if(key == VALID_MOVE){
    return 1;
  }	       
  if(key == PAWN_ODD_CAP_UP){
    if(game.board[m.to] < 0 || game.en_passant == m.to){
      return 1;
    }else{
      return 0;
    }
  }else if(key == PAWN_ODD_CAP_DOWN){
    if(game.board[m.to] > 0 || game.en_passant == m.to){
      return 1;
    }else{
      return 0;
    }
  }else if(key == PAWN_EVEN_CAP_UP ){
    if(game.board[m.to] < 0|| game.en_passant == m.to){
      return 1;
    }else{
      return 0;
    }
  }else if(key == PAWN_EVEN_CAP_DOWN ){
    if(game.board[m.to] > 0|| game.en_passant == m.to ){
      return 1;
    }else{
      return 0;
    }
  }else if(key == PAWN_NORMAL_UP){
    if(game.board[m.to] == 0){
      return 1;
    }else{
      return 0;
    }
  }else if(key == PAWN_NORMAL_DOWN){
    if(game.board[m.to] == 0){
      return 1;
    }else{
      return 0;
    }
  }else if(key == PAWN_DOUBLE_UP){
    if(game.board[m.to] == 0 && game.board[m.to+8] == 0){
      return 1;
    }else{
      return 0;
    }
  }else if(key == PAWN_DOUBLE_DOWN){
    if(game.board[m.to] == 0 && game.board[m.to-8] == 0){
      return 1;
    }else{
      return 0;
    }
  }else if(key == RANK_RIGHT){
    if(game.board[m.from] < 0){
      if(game.board[m.to] < 0){
	return 0;
      }
    }else{
      if(game.board[m.to] > 0){
	return 0;
      }
    }
    for(i = m.from+1;i < m.to;i++){
      if(game.board[i] != 0){
	return 0;
      }
    }
    return 1;
  }else if(key == RANK_LEFT){
    if(game.board[m.from] < 0){
      if(game.board[m.to] < 0){
	return 0;
      }
    }else{
      if(game.board[m.to] > 0){
	return 0;
      }
    }
    for(i = m.from-1;i > m.to;i--){
      if(game.board[i] != 0){
	return 0;
      }
    }
    return 1;
  }else if(key == RANK_LEFT){
    if(game.board[m.from] < 0){
      if(game.board[m.to] < 0){
	return 0;
      }
    }else{
      if(game.board[m.to] > 0){
	return 0;
      }
    }
    for(i = m.from-1;i > m.to;i--){
      if(game.board[i] != 0){
	return 0;
      }
    }
    return 1;
  }else if(key == FILE_UP){
    if(game.board[m.from] < 0){
      if(game.board[m.to] < 0){
	return 0;
      }
    }else{
      if(game.board[m.to] > 0){
	return 0;
      }
    }
    for(i = m.from-8;i > m.to;i-=8){
      if(game.board[i] != 0){
	return 0;
      }
    }
    return 1;
  }else if(key == FILE_DOWN){
    if(game.board[m.from] < 0){
      if(game.board[m.to] < 0){
	return 0;
      }
    }else{
      if(game.board[m.to] > 0){
	return 0;
      }
    }
    for(i = m.from+8;i < m.to;i+=8){
      if(game.board[i] != 0){
	return 0;
      }
    }
    return 1;
  }else if(key == ODD_UP){
    if(game.board[m.from] < 0){
      if(game.board[m.to] < 0){
	return 0;
      }
    }else{
      if(game.board[m.to] > 0){
	return 0;
      }
    }
    for(i = m.from-7;i > m.to;i-=7){
      if(game.board[i] != 0){
	return 0;
      }
    }
    return 1;
  }else if(key == ODD_DOWN){
    if(game.board[m.from] < 0){
      if(game.board[m.to] < 0){
	return 0;
      }
    }else{
      if(game.board[m.to] > 0){
	return 0;
      }
    }
    for(i = m.from+7;i < m.to;i+=7){
      if(game.board[i] != 0){
	return 0;
      }
    }
    return 1;
  }else if(key == EVEN_UP){
    if(game.board[m.from] < 0){
      if(game.board[m.to] < 0){
	return 0;
      }
    }else{
      if(game.board[m.to] > 0){
	return 0;
      }
    }
    for(i = m.from-9;i > m.to;i-=9){
      if(game.board[i] != 0){
	return 0;
      }
    }
    return 1;
  }else if(key == EVEN_DOWN){
    if(game.board[m.from] < 0){
      if(game.board[m.to] < 0){
	return 0;
      }
    }else{
      if(game.board[m.to] > 0){
	return 0;
      }
    }
    for(i = m.from+9;i < m.to;i+=9){
      if(game.board[i] != 0){
	return 0;
      }
    }
    return 1;
  }else if(key == POSSIBLY_CASTLE){
    if(game.to_move == WHITE){
      my_top = w_castles(my_top);
    }else{
      my_top = b_castles(my_top);
    }
    while(my_top > &(stack[0])){
      --my_top;
      if((*my_top).special == m.special){
	return 1;
      }
    }
    return 0;
  }else if(key == KNIGHT_MOVE){
    if(game.board[m.from] < 0){
      if(game.board[m.to] < 0){
	return 0;
      }
    }else{
      if(game.board[m.to] > 0){
	return 0;
      }
    }
    return 1;
  }else {
    error("Can't get here!");
  }
  return 0;
}


