/*
************************************************************************************
*                                                                                  * 
*  Copyright (c) 2004, Filipe Maia                                                 *
*   All rights reserved.                                                           *
*                                                                                  *
*  Redistribution and use in source and binary forms, with or without              *
*  modification, are permitted provided that the following conditions are met:     *
*    -Redistributions of source code must retain the above copyright notice, this  *
*     list of conditions and the following disclaimer.                             *
*    -Redistributions in binary form must reproduce the above copyright notice,    *
*     this list of conditions and the following disclaimer in the documentation    *
*     and/or other materials provided with the distribution.                       *
*    -The names of its contributors may not be used to endorse or promote          *
*     products derived from this software without specific prior written           *
*     permission.                                                                  *
*                                                                                  *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"     *
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       *
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  *
*  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE     *
*  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL      *
*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR      *
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   *
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.            *
*                                                                                  *
************************************************************************************
*/

#include <bitboard.h>
#include <gen_moves.h>
#include <chess_const.h>
#include <init.h>
#include <math_util.h>
#include <zobrist.h>




BitBoard zobrist[6][2][64];
BitBoard castle_zob[2][2];
BitBoard en_passant_zob[65];
BitBoard change_side_zob;

  
/* This should be the same as crafty now*/
void init_zobrist(){
  int i,j,k;
  for(k = 0;k<64;k++){
    /* I have to translate the square from 
     crafty notation to elturco notation */
    zobrist[PAWN-1][WHITE][56-(k&56)+(k&7)] = random_64();
    zobrist[PAWN-1][BLACK][56-(k&56)+(k&7)] = random_64();
    zobrist[KNIGHT-1][WHITE][56-(k&56)+(k&7)] = random_64();
    zobrist[KNIGHT-1][BLACK][56-(k&56)+(k&7)] = random_64();
    zobrist[BISHOP-1][WHITE][56-(k&56)+(k&7)] = random_64();
    zobrist[BISHOP-1][BLACK][56-(k&56)+(k&7)] = random_64();
    zobrist[ROOK-1][WHITE][56-(k&56)+(k&7)] = random_64();
    zobrist[ROOK-1][BLACK][56-(k&56)+(k&7)] = random_64();
    zobrist[QUEEN-1][WHITE][56-(k&56)+(k&7)] = random_64();
    zobrist[QUEEN-1][BLACK][56-(k&56)+(k&7)] = random_64();
    zobrist[KING-1][WHITE][56-(k&56)+(k&7)] = random_64();
    zobrist[KING-1][BLACK][56-(k&56)+(k&7)] = random_64();
  }
  for(i = 0;i<2;i++){
    for(j = 0;j<2;j++){
      castle_zob[j][i] = random_64();
    }
  }
  
  en_passant_zob[0] = 0;
  for(i = 1;i<65;i++){
    en_passant_zob[i] = random_64();
  }
  for(i = 0;i<2;i++){
    wtm_random[i] = random_64();
  }
  change_side_zob = random_64();
}


BitBoard gen_pawn_key(){
  BitBoard a = 0;
  int c;
  int i;
  for(i = 0;i<64;i++){
    if(game.board[i] == b_pawn){
      c = BLACK;
      a ^= zobrist[PAWN-1][c][i];
    }else if(game.board[i] == w_pawn){
      c = WHITE;
      a ^= zobrist[PAWN-1][c][i];
    }
  }
  return a;
}


BitBoard gen_zob_key(){
  BitBoard a = 0;
  int n;
  int c;
  int i;
  for(i = 0;i<64;i++){
    /*we only take into account the position where there
     is a piece. none == 0*/
    if((n = game.board[i]) < 0){
      c = BLACK;
      n = -n;
      a ^= zobrist[n-1][c][i];
    }else if(n){
      c = WHITE;
      a ^= zobrist[n-1][c][i];
    }
  }
  if((game.new_castle & W_SMALL_CASTLE) == 0)
    a ^= castle_zob[WHITE][0];    
  if((game.new_castle & W_BIG_CASTLE) == 0)
    a ^= castle_zob[WHITE][1];    
  if((game.new_castle & B_SMALL_CASTLE) == 0)
    a ^= castle_zob[BLACK][0];    
  if((game.new_castle & B_BIG_CASTLE) == 0)
    a ^= castle_zob[BLACK][1];    

  if(game.en_passant){
    a ^= en_passant_zob[(int)game.en_passant];
  }
  if(game.to_move == BLACK)
    a ^= change_side_zob;
  return a;
}
